package rocketBugsRender;
/*
	BatchHist... gui frame that controls batch histogram creation
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.text.*;
import java.util.*;


public class BatchHist extends JFrame implements ChangeListener, ActionListener, WindowListener, ComponentListener, KeyListener, MouseListener, MouseMotionListener, AdjustmentListener {
	RocketBugsRender MP;		// a reference back to the parent JFrame of this panel
 	Dimension frameDim = new Dimension(400,400);
 	JFileChooser opener = new JFileChooser();
 	File batchFolder;
 	String batchPath;
 	String [] batchList;
 	DataSet inSet;
 	DataColumn pathCol,timeCol,tethersCol;
 	EventTracker eventT = new EventTracker();
 	StepAnalyzer pass1 = new StepAnalyzer();
 	StepAnalyzer pass2 = new StepAnalyzer();
 	boolean batchingSteps, batchingEvents, batchingPauses; 	// flags indicate what operation we're performing in batch
 	boolean eventsAreSet = false;
 	boolean pass1On = true;		// use pass1 for smoothing flag
 	int startTime = 2;
	int stopTime = (int)1e6;	// big number

 	// misc gui
 	JButton setEventsB;
 	JButton setPassB;
 	JButton newBatchB;
 	JComboBox batchOptionsBox;
 	JCheckBox smoothOn;
 	
 	static final int startTimeRange = 10; // (up to 10 seconds)
	static final int stopTimeRange = 60; // (up to 60 seconds)
	static String startTimeString = "Start Time =";
	static String stopTimeString = "Stop Time = ";
	JSlider startTimeSlide = new JSlider (JSlider.HORIZONTAL,0,startTimeRange,startTime);
	JSlider stopTimeSlide = new JSlider (JSlider.HORIZONTAL,0,stopTimeRange,stopTimeRange);
	JLabel startTimeLabel = new JLabel (startTimeString + String.valueOf(startTime));
	JLabel stopTimeLabel = new JLabel (stopTimeString + "eof");
 	
 	// panel components
 	JPanel funcPanel = new JPanel();
 	JPanel contPanel = new JPanel();
 	JPanel paramPanel = new JPanel();
 	JPanel optionsPanel = new JPanel();
 	JPanel timePanel = new JPanel();
 
	public BatchHist () {
		setTitle (" Batch Histogram For Steps or Events ");
		setSize(frameDim);
		setResizable(true);
		setLocation(50,0);
		
		// make panels
		makeFuncPanel();
		makeTimePanel();
		//makeContPanel();
		optionsPanel.add(new JLabel(" **** Load Files **** "));
		
		startTimeSlide.addChangeListener(this);
		stopTimeSlide.addChangeListener(this);
		
		addKeyListener(this);
		addComponentListener(this);
		addMouseListener(this);
		addMouseMotionListener(this);
		addWindowListener(this);
		
		initialize();
	}
	
	public void initialize () {
		show();
		getContentPane().removeAll();
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add (funcPanel, BorderLayout.SOUTH);
		getContentPane().add (paramPanel, BorderLayout.CENTER);
		getContentPane().add (optionsPanel, BorderLayout.NORTH);
		//getContentPane().add (contPanel, BorderLayout.SOUTH);
		show();
		requestFocus();
	}
	
	public void newBatch() {
		switch (batchOptionsBox.getSelectedIndex()) {
			case 0:
				batchingEvents = false;
				batchingSteps = true;
				batchingPauses = true;
				break;
			case 1:
				batchingEvents = true;
				batchingSteps = false;
				batchingPauses = false;
				break;
			case 2:
				batchingEvents = true;
				batchingSteps = true;
				batchingPauses = true;
				break;
		}
		batchFolder = null;
		loadBatchSet();
		if (batchFolder != null) {
			//batchSet();
			setPassParams();
		}
	}
	
	/*public void newPlateauBatch () {
		batchFolder = null;
		loadBatchSet();
		if (batchFolder != null) {
			batchPlateaus();
		}
	}
	
	public void newEventProfileBatch () {
		batchFolder = null;
		loadBatchSet();
		if (batchFolder != null) {
			batchEvents();
		}
	}*/
		
	public void loadBatchSet() {
		opener.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int fileQuery = opener.showOpenDialog(this);
		if (fileQuery == JFileChooser.APPROVE_OPTION) {
            batchFolder = opener.getSelectedFile();
            talkln ("Batch histogram with folder " + batchFolder.getAbsolutePath());
        } else {
            talkln ("Open command cancelled by user");
            return;
        }
        batchPath = batchFolder.getAbsolutePath();
        batchList = batchFolder.list();
        
	}
	
	public void loadSet(String curFile) {
		inSet = new DataSet (batchPath + "/" + curFile);
		timeCol = getTimeCol();
		pathCol = getPathCol();
		tethersCol = getTethersCol();
	}
	
	public void loadEventNames() {
		for (int i=0;i<batchList.length;i++) {
			if (batchList[i].endsWith("ListeriaInfo")) {
				String [] eventNames = FileOps.getDataNamesOnly(batchPath + "/" + batchList[i]);
				inSet = new DataSet(eventNames.length,"eventNamesOnly",eventNames);
				return;
			}
		}
	}
	
	public void batchSet () {
		Env.batching = true;
		Env.dataColumnsOn = eventT.onList;
		// count files to process
		int batchCt = 0;
		int curStep = 0;
		double simTime = 0;
		double curSimTime;
		double aveVeloc = 0;
		double pauseTimeSum = 0;
		for (int i=0;i<batchList.length;i++) { if (batchList[i].endsWith("ListeriaInfo")) { batchCt++; } }
		HistogramPlus pauseHisto = null;
		HistogramPlus stepHisto = null;
		HistogramPlus [] eventHistos = null;
		HistogramPlus [] tetherCor = new HistogramPlus [2];
		HistogramPlus velocHisto = null;
		for (int i=0;i<batchList.length;i++) {
			if (batchList[i].endsWith("ListeriaInfo")) {
				talkln ("");
				talkln ("** working on file " + String.valueOf(curStep+1) + "/" + String.valueOf(batchCt));
				loadSet (batchList[i]);
				
				if (pass1On) {
					// ** first pass... smooth line
					pass1.setColumns(pathCol,timeCol);
					pass1.setStartAndStop(startTime,stopTime);
					pass1.setTethersCol(tethersCol);
					pass1.fitLines();
					pass1.mergeLines();
				}
				
				// ** second pass... fit "flats"
				if (pass1On) {
					pass2.setColumns(pass1.outCol,timeCol);
				} else {
					pass2.setColumns(pathCol,timeCol);
				}
				pass2.setStartAndStop(startTime,stopTime);
				pass2.setTethersCol(tethersCol);
				pass2.outCol.clearLines();
				pass2.fitLines3();
				
				// *** store sim time and ave veloc
				curSimTime = timeCol.data[pass2.stopIndex] - timeCol.data[pass2.startIndex];
				simTime += curSimTime;
				aveVeloc += (pathCol.data[pass2.stopIndex] - pathCol.data[pass2.startIndex])/curSimTime;
				
				
				
				if (pass2.outCol.lineCt != 0) {	// don't continue if no lines fit
					// ** make plateaus
					//pass2.curDisJ = 0.0010;	// (m) disjoint distance
					pass2.stepCol.clearLines();
					pass2.makePlateaus();
					pass2.filterPlateaus();
					
					if (batchingSteps) {
						HistogramPlus curStepsH = pass2.makeStepHistogram (pass2.stepCol);
						if (curStep==0) {
							stepHisto = curStepsH;
							stepHisto.name = "step size (m)";
						} else {
							stepHisto.addHisto(curStepsH);
						}
						//plateauHistos[curStep] = pass2.makeHistogram(pass2.stepCol, batchPath, batchList[i] + "-steps");
					}
					
					if (batchingPauses) {
						HistogramPlus curPause = pass2.makePauseHistogram (pass2.stepCol);
						if (curStep==0) {
							pauseHisto = curPause;
							pauseHisto.name = "pause length (s)";
						} else {
							pauseHisto.addHisto(curPause);
						}
					}
					
					if ((batchingSteps) | (batchingPauses)) {
						HistogramPlus curVelocs = pass2.makeVelocHistogram(pass2.stepCol);
						if (curStep == 0) {
							velocHisto = curVelocs;
							velocHisto.name = "velocities (m/s)";
						} else {
							velocHisto.addHisto(curVelocs);
						}
					}
					
					if (batchingEvents) {
						eventT.setSet(inSet,pass2.stepCol,timeCol);
						HistogramPlus [] curEvents = eventT.makeEventProfiles();
						//HistogramPlus [] curTetherCor = eventT.pauseCTethers(pass2.startIndex);
						if (curStep==0) { 
							eventHistos = curEvents; 
							//tetherCor = curTetherCor;
						} else {
							for (int k=0;k<eventHistos.length;k++) {
								eventHistos[k].addHisto(curEvents[k]);
							}
							// correlation histograms
							/*for (int k=0;k<tetherCor.length;k++) {
								tetherCor[k].addHisto(curTetherCor[k]);
							}*/
						}
					}
					
					//*** Store total pause time
					pauseTimeSum += pass2.sumPauseTime(pass2.stepCol);
				}
				curStep++;
				
				// try and clear some memory
				inSet.nullLinks();
				inSet = null;
				timeCol.nullLinks();
				timeCol = null;
				pathCol.nullLinks();
				pathCol = null;
			
				System.gc();
			}
		}	
		
	
		if (batchingSteps) { 
			stepHisto.infoS = makeInfoStrings(curStep,stepHisto.ptCt,simTime,aveVeloc/curStep,pauseTimeSum);
			FileOps.mkHistogramFile(stepHisto); 
		}
		if (batchingPauses) { 
			pauseHisto.infoS = makeInfoStrings(curStep,pauseHisto.ptCt,simTime,aveVeloc/curStep,pauseTimeSum);
			FileOps.mkHistogramFile(pauseHisto); 
		}
		
		if ((batchingSteps) | (batchingPauses)) {
			velocHisto.infoS = makeInfoStrings(curStep,velocHisto.ptCt,simTime,aveVeloc/curStep,pauseTimeSum);
			FileOps.mkHistogramFile(velocHisto);
		}
		
		/*if ((batchingSteps) & (batchingPauses)) {
			stepHisto.infoS = makeInfoStrings(curStep,stepHisto.ptCt,simTime,aveVeloc/curStep);
			pauseHisto.infoS = makeInfoStrings(curStep,pauseHisto.ptCt,simTime,aveVeloc/curStep);
			FileOps.mkHistogramFile(new HistogramPlus [] {pauseHisto,stepHisto}); 
		}*/

		if (batchingEvents) { 
			eventHistos[0].infoS = makeInfoStrings(curStep,eventHistos[0].passCt,simTime,aveVeloc/curStep,pauseTimeSum);
			FileOps.mkHistogramFile(eventHistos); 
			// correlation histos
			/*tetherCor[0].infoS = eventHistos[0].infoS;
			FileOps.mkHistogramFile(tetherCor);*/
		}
		
		talkln ("");
		talkln ("Done with Batch *****");
		Env.batching = false;
	}
	
	public String [] makeInfoStrings(int numFiles, int numPasses, double totSimTime, double aveVeloc, double pauseTimeSum) {
		Date timeNow = new Date();
		String [] infoS = new String [11];
		infoS[0] = "Batch info: " + batchPath + " run on " + timeNow.toString();
		infoS[1] = "Start time: " + String.valueOf(startTime) + " Stop time: " + String.valueOf(stopTime);
		if (pass1On) {
			infoS[2] = "pass1: lap=" + String.valueOf(pass1.curLap) + ", X^2=" + String.valueOf(pass1.curChiSq);
		} else {
			infoS[2] = "pass1 off";
		}
		infoS[3] = "pass2: minPause=" + String.valueOf(pass2.curMinPause) + ", veloc=" + String.valueOf(pass2.curVeloc) + ", X^2=" + String.valueOf(pass2.curChiSq);
		infoS[4] = "pass3: disJoint distance=" + String.valueOf(pass2.curDisJ);
		if (batchingEvents) {
			infoS[5] = "pause duration selection criteria:" + String.valueOf(eventT.minPauseTime) + " - " + String.valueOf(eventT.maxPauseTime) + " seconds";
		} else {
			infoS[5] = "pause duration selection criteria: N/A";
		}
		infoS[6] = "number of files processed = " + String.valueOf(numFiles);
		infoS[7] = "number of plateaus/passes = " + String.valueOf(numPasses);
		infoS[8] = "simulation time processed = " + String.valueOf(totSimTime) + " seconds";
		infoS[9] = "average velocity = " + String.valueOf(aveVeloc) + " m/s";
		infoS[10] = "total pause time = " + String.valueOf(pauseTimeSum) + " s";
		
		return infoS;
	}
		
	
	/*public void batchPlateaus() {
		batchingEvents = false;
		batchingSteps = true;
		batchSet();
	}
	
	public void batchEvents () {
		batchingEvents = true;
		batchingSteps = false;
		batchSet();
	}*/
	
	public void setPassParams () {
		paramPanel.removeAll();
		paramPanel.setLayout(new BorderLayout());
		
		JPanel pass1Pan = new JPanel();
		pass1Pan.setLayout(new GridLayout(1,2));
		pass1Pan.add(pass1.sliPanel);
		pass1Pan.add(pass1.labPanel);
		
		JPanel pass2Pan = new JPanel();
		pass2Pan.setLayout(new GridLayout(1,2));
		pass2Pan.add(pass2.sliPanel);
		pass2Pan.add(pass2.labPanel);
		
		JLabel pass1Label = new JLabel("Pass1:");
		pass1Label.setFont(Env.bigFont);
		pass1Label.setVerticalAlignment(JLabel.BOTTOM);
		
		JLabel pass2Label = new JLabel("Pass2:");
		pass2Label.setFont(Env.bigFont);
		pass2Label.setVerticalAlignment(JLabel.BOTTOM);
		
		JPanel pass1Head = new JPanel(new GridLayout(1,2));
		smoothOn = new JCheckBox("pass1 on");
		smoothOn.setActionCommand("SmoothChanged");
		smoothOn.addActionListener(this);
		smoothOn.setSelected(pass1On);
		pass1Head.add(pass1Label);
		pass1Head.add(smoothOn);
		
		JPanel p1 = new JPanel();
		p1.setLayout(new BorderLayout());
		p1.add(pass1Head,BorderLayout.NORTH);
		p1.add(pass1Pan,BorderLayout.CENTER);
		
		JPanel p2 = new JPanel();
		p2.setLayout(new BorderLayout());
		p2.add(pass2Label,BorderLayout.NORTH);
		p2.add(pass2Pan,BorderLayout.CENTER);
		
		JPanel pFinal = new JPanel(new GridLayout(2,1));
		pFinal.add(p1);
		pFinal.add(p2);
		
		paramPanel.add(timePanel,BorderLayout.NORTH);
		paramPanel.add(pFinal,BorderLayout.CENTER);
		
		optionsPanel.removeAll();
		optionsPanel.add(new JLabel("**** Set Pass Parameters ****"));
		setPassB.setEnabled(true);
		setEventsB.setEnabled(false);
		newBatchB.setEnabled(false);
		initialize();
	}
	
	public void setEventNames () {
		loadEventNames();
		eventT.initControlStrings();
		eventT.setDataSet(inSet);
		eventT.makeControlPanel();
		eventT.setDataPanel();
		paramPanel.removeAll();
		paramPanel.setLayout(new BorderLayout());
		paramPanel.add(eventT.dataPanel,BorderLayout.CENTER);
		paramPanel.add(eventT.controlPanel,BorderLayout.NORTH);
		optionsPanel.removeAll();
		optionsPanel.add(new JLabel("**** Set Event Parameters ****"));
		eventT.workB.setEnabled(false);
		setPassB.setEnabled(false);
		setEventsB.setEnabled(true);
		newBatchB.setEnabled(false);
		initialize();
	}
	
	public void startEventBatch() {
		eventT.parseGUI();
		batchSet();
	}
	
	private DataColumn getTimeCol () {
		return inSet.dataTable[0];
	}
	
	private DataColumn getPathCol () {
		return inSet.dataTable[1];
	}
	
	private DataColumn getTethersCol () {
		for (int i=0;i<inSet.dims[0];i++) {
			if (inSet.dataTable[i].name.equals("#effTethers")) {
				tethersCol = inSet.dataTable[i];
			}
		}
		return tethersCol;
	}
	
	public void actionPerformed (ActionEvent e) {
		String act = e.getActionCommand();
		
		if (act.equals("New Batch")) {
			newBatch();
		}
		
		if (act.equals("Set Pass")) {
			if (batchingEvents) {
				setEventNames();
			} else {
				batchSet();
			}
		}
		
		if (act.equals("Set Events")) {
			startEventBatch();
		}
		
		if (act.equals("SmoothChanged")) {
			pass1On = smoothOn.isSelected();
			if (pass1On) {
				pass1.allSlidesEnabled();
				pass1.updateGUI();
			} else {
				pass1.allSlidesDisabled();
			}
		}
	}
	
	public void stateChanged(ChangeEvent e) {
        JSlider source = (JSlider)e.getSource();
        
        if (source.equals(startTimeSlide)) {
        	startTime = (int)(startTimeSlide.getValue());
        	startTimeLabel.setText(startTimeString + String.valueOf(startTime));
        }
        
        if (source.equals(stopTimeSlide)) {
        	stopTime = (int)(stopTimeSlide.getValue());
        	stopTimeLabel.setText(stopTimeString + String.valueOf(stopTime));
        	if (stopTime == stopTimeRange) {
        		stopTime = (int)1e6;
        		stopTimeLabel.setText(stopTimeString + "eof");
        	}
        }
    }
	
	public void adjustmentValueChanged (AdjustmentEvent e) {
	}
	
	
	public void windowActivated (WindowEvent e) {}
	public void windowClosing (WindowEvent e) {}
	public void windowDeactivated (WindowEvent e) {}
	public void windowDeiconified (WindowEvent e) {}
	public void windowIconified (WindowEvent e) {}
	public void windowOpened (WindowEvent e) {}
	public void windowClosed (WindowEvent e) {}
	
	public void keyReleased (KeyEvent e) {}
	public void keyTyped (KeyEvent e) {}
	public void keyPressed (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
	}
	
	public void mouseClicked (MouseEvent e) {}
	public void mouseEntered (MouseEvent e) {}
	public void mouseExited (MouseEvent e) {}
	public void mousePressed (MouseEvent e) {
		int xPos = e.getX();
		int yPos = e.getY();
	}
	
	public void mouseReleased (MouseEvent e) {}
	public void mouseDragged (MouseEvent e) {}
	public void mouseMoved (MouseEvent e) {}
	
	public void componentHidden (ComponentEvent e) {}
	public void componentShown (ComponentEvent e) {}
	public void componentMoved (ComponentEvent e) {}
	public void componentResized (ComponentEvent e) {}
	
	private void makeTimePanel () {
		timePanel = new JPanel();
		timePanel.setLayout(new GridLayout(2,2));
		timePanel.add(startTimeSlide);
		timePanel.add(startTimeLabel);
		timePanel.add(stopTimeSlide);
		timePanel.add(stopTimeLabel);
	}
	
	public void makeFuncPanel() {
		newBatchB = new JButton ("1. Select Files");
		newBatchB.setFont(Env.controlFont);
		newBatchB.setActionCommand("New Batch");
		newBatchB.setToolTipText("Begin a new batch, save histogram");
		newBatchB.addActionListener(this);
		newBatchB.setSize(10,10);
		
		/*JButton batchEventButton = new JButton ("Batch Events");
		batchEventButton.setFont(Env.controlFont);
		batchEventButton.setActionCommand("New Event Batch");
		batchEventButton.setToolTipText("Begin a new batch, save event histogram");
		batchEventButton.addActionListener(this);
		batchEventButton.setSize(10,10);*/
		
		String [] batchOptions = {"steps & pauses", "events", "steps, pauses & events"};
		batchOptionsBox = new JComboBox(batchOptions);
		batchOptionsBox.setSelectedIndex(1);
		batchOptionsBox.setFont(Env.controlFont);
		batchOptionsBox.setActionCommand("BatchOptions");
		batchOptionsBox.setToolTipText("Select type of batch job");
		batchOptionsBox.addActionListener(this);
		
		setEventsB = new JButton ("3. Set Events");
		setEventsB.setActionCommand("Set Events");
		setEventsB.setFont(Env.controlFont);
		setEventsB.setToolTipText("Once events are set, press to continue");
		setEventsB.addActionListener(this);
		setEventsB.setEnabled(false);
		
		setPassB = new JButton ("2. Set Pass Params");
		setPassB.setActionCommand("Set Pass");
		setPassB.setFont(Env.controlFont);
		setPassB.setToolTipText("Once pass params are set, press to continue");
		setPassB.addActionListener(this);
		setPassB.setEnabled(false);
		
		funcPanel.setLayout(new GridLayout(1,4));
		funcPanel.add(batchOptionsBox);
		funcPanel.add(newBatchB);
		funcPanel.add(setPassB);
		funcPanel.add(setEventsB);
		funcPanel.setPreferredSize(new Dimension(200,30));
	}
	
/*	public void makeContPanel() {
		setEventsB = new JButton ("Set Events");
		setEventsB.setActionCommand("Set Events");
		setEventsB.setToolTipText("Once events are set, press to continue");
		setEventsB.addActionListener(this);
		setEventsB.setEnabled(false);
		
		setPassB = new JButton ("Set Pass Params");
		setPassB.setActionCommand("Set Pass");
		setPassB.setToolTipText("Once pass params are set, press to continue");
		setPassB.addActionListener(this);
		setPassB.setEnabled(false);
		
		contPanel.setLayout(new FlowLayout());
		contPanel.add(setPassB);
		contPanel.add(setEventsB);
	}*/
	
	private void talk (String info) {
		System.out.print(info);
	}
	
	private void talkln (String info) {
		System.out.println(info);
	}
	
	
}
	