package rocketBugsRender;
/* 
	ButtonTable .... a class to access a mess of ButtonLists
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.text.*;

public class ButtonTable extends JPanel {
	int maxCols;
	ButtonList [] boxList;
	int setCt = 0;
	boolean mExclusive = false;
	ButtonGroup bGroup;
	
	public ButtonTable (int maxCols, boolean mX) {
		this.maxCols = maxCols;
		this.mExclusive = mX;
		boxList = new ButtonList [maxCols];
		if (mExclusive) { bGroup = new ButtonGroup(); }
	}
	
	public void updateLayout () {
		this.removeAll();
		this.setLayout(new GridLayout(1,setCt));
		for (int i=0;i<setCt;i++) {
			this.add(boxList[i].lBox);
		}
			
	}
	
	public void addList (ButtonList newList) {
		addList(newList,null,null);
	}
	
	public void addList (ButtonList newList, String actionC, ActionListener listener) {
		boxList[setCt] = newList;
		newList.index = setCt;
		if (mExclusive) { newList.addToButtonGroup(bGroup); }	// add to bGroup if exclusive group
		if (actionC != null) { newList.setActionCommand(actionC,listener); }
		setCt++;
		updateLayout();
	}
	
	public void ammendList (ButtonList amList) {
		ammendList(amList,null,null);
	}
	
	public void ammendList (ButtonList amList, String actionC, ActionListener listener) {
		int index = getIndex(amList);
		if (index == -1) {
			talkln ("Error ammending ButtonList in ButtonTable");
			return;
		}
		boxList[index] = amList;
		if (mExclusive) { amList.addToButtonGroup(bGroup); }	// add to bGroup if exclusive group
		if (actionC != null) { amList.setActionCommand(actionC,listener); }
		updateLayout();
	}
	
	public void rmList (ButtonList rmList) {
		if (mExclusive) { rmList.rmFromButtonGroup(bGroup); } 	// rm from bGroup
		int index = getIndex(rmList);
		if (index == -1) {
			talkln ("Error removing ButtonList from ButtonTable");
			return;
		}
		if (index == setCt-1) { 
			setCt += -1; 
			return;
		}
		shiftLeft(index,setCt);
		rmList.index = -1;
	}
	
	public void shiftLeft (int start, int stop) {
		for (int i=start;i<stop-1;i++) {
			boxList[i] = boxList[i+1];
			boxList[i].index = i;
		}
	}
	
	public int getIndex(ButtonList list) {
		for (int i=0;i<setCt;i++) {
			if (list.equals(boxList[i])) {
				return i;
			}
		}
		// didn't get an index from object matching.... use ButtonList index
		return list.index;
	}
		
	
	public AbstractButton getSelectedButton () {
		if (!mExclusive) { talkln ("ERROR.. selected button not defined for non-exclusive button table"); }
		for (int i=0;i<setCt;i++) {
			//CODE INCOMPLETE
		}
		return null;
	}
	
	public void updateColors (Color newColor) {
		for (int i=0;i<setCt;i++) {
			boxList[i].updateColors (newColor);
		}
	}
		
	private void talk (String info) {
		System.out.print(info);
	}
	
	private void talkln (String info) {
		System.out.println(info);
	}

	
}