package rocketBugsRender;
/*
	DataColumn ... just a simple double array with some extras and special methods
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;

public class DataColumn {
	DataSet set;	// set to which this column belongs, if any
	int rows;
	double [] data;
	String name;
	double [] globalRange = new double[2];
	int [] curRowRange = new int[2];
	double range;
	double yPixFactor, xPixFactor;
	double yPixOffset;
	Pix2DColumn pixsCol;
	Axis axis;
	PlotProps pProps;
	Dimension pixels;
	
	boolean analyze = false;	// this data set for analysis
	
	// graphing properties 
	boolean onY = false;		// plot on y-axis
	boolean onX = false;		// plot on x-axis
	
	public DataColumn (int rows, String name, DataSet set) {
		this.set = set;
		this.rows = rows;
		data = new double [rows];
		this.name = name;
		setGlobalRange(0,rows);
		createObjects();
	}
	
	public DataColumn (int rows, DataSet set) {
		this.set = set;
		this.rows = rows;
		data = new double[rows];
		createObjects();
	}
		
	public DataColumn (int rows) {
		this.rows = rows;
		data = new double [rows];
		createObjects();
	}
	
	public DataColumn () {
		createObjects();
	}
	
	public void nullLinks() {
		set = null;
		data = null;
		globalRange = null;
		pixsCol = null;
		axis = null;
		pProps = null;
		pixels = null;
	}
	
	public void createObjects () {
		pProps = new PlotProps(this);
		axis = new Axis(this);
	}
	
	public String getName () {
		if (name != null) { return name; } else { return "?"; }
	}
	
	public String getSetName() {
		if (set != null) {
			return set.getName();
		} else {
			return " ";
		}
	}
	
	public void setName (String name) {
		this.name = name;
	}
	
	public void setColor (Color newColor ) {
		pProps.color = newColor;
		axis.color = newColor;
	}
	
	public void setGlobalRange (int start, int stop) {
		if (start<0) { start = 0; }
		if (stop>rows) { stop = rows; }
		if (start == stop) { start = start-1; }
		double max = data[start];
		double min = data[start];
		for (int i=start;i<stop;i++) {
			if (data[i]>max) { max = data[i]; }
			if (data[i]<min) { min = data[i]; }
		}
		globalRange[0] = min;
		globalRange[1] = max;
		
	}
	
	public void setLocalRange (int start, int stop) {
		if (start<0) { start = 0; }
		if (stop>rows) { stop = rows; }
		if (start == stop) { start = start-1; }
		double max = data[start];
		double min = data[start];
		for (int i=start;i<stop;i++) {
			if (data[i]>max) { max = data[i]; }
			if (data[i]<min) { min = data[i]; }
		}
		pProps.min = min;
		pProps.max = max;
		
	}
	
	public double rangeAve () {
		double sumAve = 0;
		for (int i=curRowRange[0];i<curRowRange[1];i++) {
			sumAve += data[i];
		}
		sumAve = sumAve/(curRowRange[1]-curRowRange[0]);
		return sumAve;
	}
	
	public double rangeSpeed (DataColumn timeCol) {
		double datRange = data[curRowRange[1]-1] - data[curRowRange[0]];
		double timeRange = timeCol.data[curRowRange[1]-1] - timeCol.data[curRowRange[0]];
		return datRange/timeRange;
	}
	
	public void setScale (Dimension pixels, int [] rowRange) {
		if (onX) {
			this.pixels = pixels;
			// first find range of values in the row range
			curRowRange[0] = rowRange[0];
			curRowRange[1] = rowRange[1];
			if (curRowRange[1] > rows) { curRowRange[1] = rows; }
			setLocalRange(curRowRange[0],curRowRange[1]);
			if (!pProps.holdRange) { range = pProps.max-pProps.min; } // don't reset range if holding scale
			if ((range == 0) | (curRowRange[0] == curRowRange[1])) {
				xPixFactor = 0;
				yPixFactor = 0;
			} else {
				// continue to use rowRange here
				yPixFactor = ((double)pixels.height)/range; 
				xPixFactor = ((double)pixels.width)/((double)(rowRange[1]-rowRange[0]));
			}
			yPixOffset = pProps.min;
			setXAxis();	// set axis values
		}
		if (onY) {
			this.pixels = pixels;
			// first find range of values in the row range
			curRowRange[0] = rowRange[0];
			curRowRange[1] = rowRange[1];
			if (curRowRange[1] > rows) { curRowRange[1] = rows; }
			// auto limits from minimum to maximum value, unless user defined
			if (!pProps.customLimits) {
				setLocalRange(curRowRange[0],curRowRange[1]);
			}
			if (pProps.zeroMin) { pProps.min = 0; }
			if (!pProps.holdRange) { range = pProps.max-pProps.min; } // reset range unless holding scale
			if ((range == 0) | (curRowRange[0] == curRowRange[1])) {
				xPixFactor = 0;
				yPixFactor = 0;
			} else {
				// continue to use rowRange NOT curRange here
				double fracPixHeight = pixels.height*pProps.pixFrac;
				yPixFactor = (fracPixHeight)/range; 
				xPixFactor = ((double)pixels.width)/((double)(rowRange[1]-rowRange[0]));
			}
			yPixOffset = pProps.min;
			setYAxis();	// set axis values
		}
	}
	
	public int getPixColCt () {
		return 1;
	}
			
	public void setYAxis () {
		axis.name = name;
		if (useFixedAxis(name)) { axis.fixedTic = true; } else { axis.fixedTic = false; }
		axis.makeTics(pProps.min,range,1,(int)(pProps.pixFrac*pixels.height));
		axis.turnOn();
	}
	
	public void setXAxis () {
		axis.name = name;
		axis.fixedTic = false;
		axis.showZero = false;
		axis.makeTics(pProps.min,range,1,pixels.width);
		axis.turnOn();
	}
	
	public int getClosestDataLoc (double valToMatch) {
		// assumes that the data is monotonically increasing
		int loc = (rows-1)/2;
		int minLoc = 0;
		int maxLoc = rows-1;
		boolean closestFound = false;
		while (!closestFound) {
			//talkln (String.valueOf(minLoc) + "," + String.valueOf(data[minLoc]) + ";" + String.valueOf(loc) + "," + String.valueOf(data[loc]) + ";" + String.valueOf(maxLoc) + "," + String.valueOf(data[maxLoc]));
			if (data[loc] < valToMatch) {
				minLoc = loc;
				loc = (loc + maxLoc)/2;
			} else {
				maxLoc = loc;
				loc = (minLoc + loc)/2;
			}
			if ((maxLoc-minLoc) <= 2) { closestFound = true; }
		}
		return loc;
	}
				
	
	private boolean useFixedAxis (String thetheName) {
		if (thetheName.equals("path.x")) { return true; }
		if (thetheName.equals("coord.y")) { return true; }
		return false;
	}
	
	public Pix2DColumn [] getPixColumns () {
		if (!onY) { return null; }
		Pix2DColumn [] pixColTab = new Pix2DColumn [1];
		pixColTab[0] = new Pix2DColumn (curRowRange[1]-curRowRange[0],this);
		pixColTab[0].setName(name);
		for (int row=curRowRange[0]; row<curRowRange[1]; row++) {
			pixColTab[0].pixs[row-curRowRange[0]] = getPix2DForPoint(row);
		}
		return pixColTab;
	}
	
	/*public Pix2DColumn [] getPixColumns () {
		if (!onY) { return null; }
		Pix2DColumn [] pixColTab = new Pix2DColumn [1];
		int pointsCt = Env.pointsToPlot;
		int rowInc = (int)((curRowRange[1]-curRowRange[0])/Env.pointsToPlot);
		if (rowInc < 2) { 
			rowInc = 1;
			pointsCt = curRowRange[1]-curRowRange[0];
		}
		pixColTab[0] = new Pix2DColumn (pointsCt+1,this);
		pixColTab[0].setName(name);
		for (int row=curRowRange[0]; row<curRowRange[1]; row+=rowInc) {
			if (row > curRowRange[1]) { row = curRowRange[1]; }
			pixColTab[0].pixs[row-curRowRange[0]] = getPix2DForPoint(row);
		}
		return pixColTab;
	} */
		
		
	public Pix2D getPix2DForPoint (int row) {
		if (row >= data.length) { return null; }
		return getPix2DForPoint (row, data[row]);
	}
	
	public Pix2D getPix2DForPoint (int row, double yVal) {
		int xPix = (int)((row - curRowRange[0])*xPixFactor);
		int yPix = (int)((yVal-yPixOffset)*yPixFactor);	// 
		yPix = -1*(yPix-pixels.height);	// flip axis values
		return new Pix2D(xPix,yPix);
	}
	
	public static DataColumn copy (DataColumn inCol) {
		DataColumn outCol = new DataColumn(inCol.rows);
		outCol.setName(inCol.name);
		for (int i=0;i<inCol.rows;i++) {
			outCol.data[i] = inCol.data[i];
		}
		return outCol;
	}
	
	public DataColumn copy (int start, int stop) {
		DataColumn outCol = new DataColumn(stop-start);
		outCol.setName(this.name);
		for (int i=0;i<outCol.rows;i++) {
			outCol.data[i] = this.data[start+i];
		}
		return outCol;
	}
	
	public void talk (String info) {
		System.out.print(info);
	}
	
	public void talkln (String info) {
		System.out.println(info);
	}
}