package rocketBugsRender;
/*
	EventTracker.java	make event histograms
*/ 

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.text.*;

public class EventTracker extends JFrame implements ChangeListener,ActionListener,KeyListener {
	DataSet inSet;		// the data to work with
	LineColumn stepCol;	// defines plateaus
	DataColumn timeCol;	// time column
	DataColumn [] eventCols;
	HistogramPlus [] eventHists;
	int dataCt;
	int eventCt;
	int alignment;
	double minPauseTime,maxPauseTime;
	double sepTime = 0;//10;	// (ms) minimum time separating adjacent pauses
	EventSpec [] eventSpecs;	// summary of gui input on events
	boolean [] onList;				// which columns are selected
	static final int ReportSumInt = 0;	// integer values for positioning and selecting output types menu
	static final int ReportNormInt = 1;
	static final int ReportAveInt = 2;
	static final int ReportScaledInt = 3;
	
	static final int AlignStart = 0;
	static final int AlignMid = 1;
	static final int AlignStop = 2;
	static final int AlignMid_Run = 3;
	
	static final int TypeActual = 0;
	static final int TypeRelative = 1;
	static final int TypeAbsolute = 2;
	static final int TypeAbsoluteRelative = 3;
	
	// gui stuff
	JPanel controlPanel = new JPanel();
	JPanel dataPanel = new JPanel();
	JButton workB;	// the start button essentially
	JLabel [] dataLabels = new JLabel[4];
	String [] labelTitles = {"Event","Data Type","Normalized By","Output Type"};
	ButtonList dataButs;
	// alignment options
	String [] alignVals = new String [4];
	JComboBox alignType;
	// data type options
	String [] dataTypeVals = new String [4];
	JComboBox [] dataTypes;
	
	JTextField [] dataNorms;
	//JMenu [] dataOutputs;
	//JMenuBar [] dataMenuBars;
	JComboBox [] dataOuts;
	String [] dataOutsVals = new String [4];
	
	JTextField maxPause,minPause;
	
	
	public EventTracker (DataSet inSet, LineColumn stepCol, DataColumn timeCol) {
		initControlStrings();
		this.inSet = inSet;
		this.stepCol = stepCol;
		this.timeCol = timeCol;
		setDataPanel();
		makeControlPanel();
		
		// frame settings
		setTitle ("Events: " + inSet.name);
		setSize(getMySize());
		setResizable(true);
		setLocation(0,0);
		
		addKeyListener(this);
		
		initialize();
		
	}
	
	public EventTracker () {
		//initControlStrings();
	}
	
	public void nullLinks () {
		inSet = null;
		stepCol = null;
		timeCol = null;
		eventCols = null;
		eventHists = null;
	}
	
	public void initControlStrings() {
		alignVals[AlignStart] = "start";
		alignVals[AlignMid] = "middle";
		alignVals[AlignStop] = "stop";
		alignVals[AlignMid_Run] = "mid-run";
		alignType = new JComboBox(alignVals);
		
		dataTypeVals[TypeActual] = "actual";
		dataTypeVals[TypeRelative] = "relative";
		dataTypeVals[TypeAbsolute] = "absolute";
		dataTypeVals[TypeAbsoluteRelative] = "absolute relative";
		
		dataOutsVals[ReportSumInt] = "sum";
		dataOutsVals[ReportNormInt] = "ave by pts";
		dataOutsVals[ReportAveInt] = "ave by passes";
		dataOutsVals[ReportScaledInt] = "scaled";
	}
	
	public class EventSpec {	// store gui input on data columns; info on how to process event
		int index;
		int valType;
		double normFactor;
		// histogramPlus output variables
		boolean reportSum,reportNorm,reportAve,reportScaled;
		
		public EventSpec (int index, int valType, double normFactor) {
			this.index = index;
			this.valType = valType;
			this.normFactor = normFactor;
		}
		
		public void resetOuts () {
			reportSum = false;
			reportNorm = false;
			reportAve = false;
			reportScaled = false;
		}
	}
		 
	
	public void setDataSet (DataSet inSet) {
		this.inSet = inSet;
	}
	
	public void setStepCol (LineColumn stepCol) {
		this.stepCol = stepCol;
	}
	
	public void setTimeCol (DataColumn timeCol) {
		this.timeCol = timeCol;
	}
	
	public void setSet (DataSet inSet, LineColumn stepCol, DataColumn timeCol) {
		setDataSet(inSet);
		setStepCol(stepCol);
		setTimeCol(timeCol);
	}
	
	private Dimension getMySize() {
		Dimension mySize = new Dimension();
		Dimension controlSize = controlPanel.getPreferredSize();
		Dimension dataSize = dataPanel.getPreferredSize();
		mySize.width = controlSize.width + dataSize.width + 40;
		mySize.height = controlSize.height + dataSize.height + 40;
		return mySize;
	}
	
	public void initialize () {
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add (controlPanel, BorderLayout.NORTH);
		getContentPane().add (dataPanel, BorderLayout.CENTER);
		show();
	}
	
	public void useAveOnly (JMenu curM) {
		curM.getItem(ReportSumInt).setSelected(false);
		curM.getItem(ReportNormInt).setSelected(false);
		curM.getItem(ReportAveInt).setSelected(true);
		curM.getItem(ReportScaledInt).setSelected(false);
	}
	
	public void useScaledOnly (JMenu curM) {
		curM.getItem(ReportSumInt).setSelected(false);
		curM.getItem(ReportNormInt).setSelected(false);
		curM.getItem(ReportAveInt).setSelected(false);
		curM.getItem(ReportScaledInt).setSelected(true);
	}
	
	public void setDefaults () {
		alignType.setSelectedIndex(2);	// default alignment by stop
		// search data for default event names
		for (int i=0;i<dataCt;i++) {
			boolean aDefault = false;
			String curCol = inSet.colNames[i];
			if (curCol.equals("path.x")) {
				aDefault = true;
				dataTypes[i].setSelectedIndex(1);
				dataNorms[i].setText("0.0027");
				//useAveOnly(dataOutputs[i]);
				dataOuts[i].setSelectedIndex(ReportAveInt);
			}
			/*if (curCol.equals("pathAng.x")) {
				aDefault = true;
				dataTypes[i].setSelectedIndex(1);
				double radToDegDivisor = Math.PI/360;
				dataNorms[i].setText(String.valueOf(radToDegDivisor));
				dataOuts[i].setSelectedIndex(ReportAveInt);
			}*/

			if (curCol.equals("actinEflex")) { aDefault = true; dataOuts[i].setSelectedIndex(ReportAveInt); }
			if (curCol.equals("actinE1")) { aDefault = true; dataOuts[i].setSelectedIndex(ReportAveInt); }
			if (curCol.equals("newTether")) { aDefault = true; dataOuts[i].setSelectedIndex(ReportAveInt); }
			if (curCol.equals("cutTether")) { aDefault = true; dataOuts[i].setSelectedIndex(ReportAveInt); }
			if (curCol.equals("forceInx")) { aDefault = true; dataOuts[i].setSelectedIndex(ReportAveInt); }
			//if (curCol.equals("forceInx-noThermal")) { aDefault = true; dataOuts[i].setSelectedIndex(ReportScaledInt); }
			if (curCol.equals("colForceInx")) { aDefault = true; dataOuts[i].setSelectedIndex(ReportAveInt); }
			if (curCol.equals("linkForceInx")) { aDefault = true; dataOuts[i].setSelectedIndex(ReportAveInt); }
			if (curCol.equals("thmKick.x")) { aDefault = true; dataOuts[i].setSelectedIndex(ReportAveInt); }
			//if (curCol.equals("thmKick/3.x")) { aDefault = true; dataOuts[i].setSelectedIndex(ReportScaledInt); }
			//if (curCol.equals("thmKick/6.x")) { aDefault = true; dataOuts[i].setSelectedIndex(ReportScaledInt); }
			//if (curCol.equals("thmTwist.x")) { aDefault = true; dataOuts[i].setSelectedIndex(ReportAveInt); }
			//if (curCol.equals("thmTwist/3.x")) { aDefault = true; dataOuts[i].setSelectedIndex(ReportScaledInt); }
			//if (curCol.equals("thmTwist/6.x")) { aDefault = true; dataOuts[i].setSelectedIndex(ReportScaledInt); }
			/*if (curCol.equals("closeTips")) { 
				aDefault = true; 
				dataTypes[i].setSelectedIndex(1);
				//useScaledOnly(dataOutputs[i]);
				dataOuts[i].setSelectedIndex(ReportAveInt);
			}*/
			if (curCol.equals("#Tethers")) { 
				aDefault = true; 
				dataTypes[i].setSelectedIndex(1);
				//useScaledOnly(dataOutputs[i]);
				dataOuts[i].setSelectedIndex(ReportAveInt);
			}
			
			
			// set button on
			if (aDefault) { dataButs.list[i].setSelected(true); }
		}
	}
	
	public void mkGuiTitles() {
		for (int i=0;i<dataLabels.length;i++) {
			dataLabels[i] = new JLabel(labelTitles[i]);
			dataLabels[i].setFont(Env.controlFont);
			dataLabels[i].setHorizontalAlignment(JLabel.CENTER);
		}
	}
	
	public void setDataPanel () {
		mkGuiTitles();
		dataCt = inSet.getColumnCt();
		dataButs = new ButtonList (inSet.name, inSet.colNames,"JCheckBox",-1);
		dataButs.setActionCommand("EventSelected",this);
		dataTypes = new JComboBox [dataCt];
		dataNorms = new JTextField [dataCt];
		//dataOutputs = new JMenu[dataCt];
		//dataMenuBars = new JMenuBar[dataCt];
		dataOuts = new JComboBox [dataCt];
		for (int i=0;i<dataCt;i++) {
			dataTypes[i] = new JComboBox(dataTypeVals);
			dataTypes[i].setFont(Env.listFont);
			//dataTypes[i].addActionListener(this);
			
			dataNorms[i] = new JTextField("none",10);
			dataNorms[i].setFont(Env.listFont);
			
			/*dataOutputs[i] = new JMenu("Output Types");
			dataOutputs[i].setFont(Env.listFont);
			dataOutputs[i].add(new JCheckBoxMenuItem("Sum",true));
			dataOutputs[i].add(new JCheckBoxMenuItem("Average by points",false),ReportNormInt);
			dataOutputs[i].add(new JCheckBoxMenuItem("Average by passes",false),ReportAveInt);
			dataOutputs[i].add(new JCheckBoxMenuItem("Scaled",false),ReportScaledInt);
			
			dataMenuBars[i] = new JMenuBar();
			dataMenuBars[i].add(dataOutputs[i]);*/
			
			dataOuts[i] = new JComboBox(dataOutsVals);
			dataOuts[i].setFont(Env.listFont);
		}
		setDefaults();
		setEnabledFields();
		updateDataPanelLayout();
	}
	
	public void updateDataPanelLayout () {
		dataPanel.removeAll();
		dataPanel.setLayout(new GridLayout(dataCt+1,4));
		for (int i=0;i<4;i++) {		// titles
			dataPanel.add(dataLabels[i]);
		}
		for (int i=0;i<dataCt;i++) {
			dataPanel.add(dataButs.list[i]);
			dataPanel.add(dataTypes[i]);
			dataPanel.add(dataNorms[i]);
			//dataPanel.add(dataMenuBars[i]);
			dataPanel.add(dataOuts[i]);
		}	
		
		// monkey with sizes
		for (int i=0;i<dataCt;i++) {
			Dimension typeSize = dataTypes[i].getPreferredSize();
			typeSize.height = 14;
			dataTypes[i].setPreferredSize(typeSize);
			Dimension normFSize = dataNorms[i].getPreferredSize();
			normFSize.height = 14;
			dataNorms[i].setPreferredSize(normFSize);
			/*Dimension outputsSize = dataOutputs[i].getPreferredSize();
			outputsSize.height = 14;
			dataOutputs[i].setPreferredSize(outputsSize);*/
			Dimension outsSize = dataOuts[i].getPreferredSize();
			outsSize.height = 14;
			dataOuts[i].setPreferredSize(outsSize);
		}
	}
	
	public void setEnabledFields () {
		for (int i=0;i<dataCt;i++) {
			if (dataButs.list[i].isSelected()) {
				dataTypes[i].setEnabled(true);
				dataNorms[i].setEnabled(true);
				//dataOutputs[i].setEnabled(true);
				dataOuts[i].setEnabled(true);
			} else {
				dataTypes[i].setEnabled(false);
				dataNorms[i].setEnabled(false);
				//dataOutputs[i].setEnabled(false);
				dataOuts[i].setEnabled(false);
			}
		}
	}
	
	public void parseGUI() {
		alignment = alignType.getSelectedIndex();
		// get minimum and maximum pause times
		Double DD = Double.valueOf(minPause.getText());
		minPauseTime = 1e-3*DD.doubleValue();	// min pause time in ms
		DD = Double.valueOf(maxPause.getText());
		maxPauseTime = 1e-3*DD.doubleValue();	// max pause time in ms
		
		makeEventSpecs();
	}
	
	public void makeEventSpecs () {
		boolean specsFailed = false;
		eventSpecs = new EventSpec[dataCt];
		onList = new boolean [dataCt];
 		eventCt = 0;
		for (int i=0;i<dataCt;i++) {
			if (dataButs.list[i].isSelected()) { 
				onList[i] = true;
				// parse data type
				int valType = dataTypes[i].getSelectedIndex();
				// parse normalization values
				double normFact = 1;
				if (!dataNorms[i].getText().equals("none")) { 
					Double normFactD = Double.valueOf(dataNorms[i].getText());
					normFact = normFactD.doubleValue();
				}
				if (Double.isNaN(normFact)) { specsFailed = true; }
				if (normFact == 0) { specsFailed = true; }
				
				EventSpec curEvent = new EventSpec(i,valType,normFact);
				// set output types
				/*JMenu curM = dataOutputs[i];
				curEvent.reportSum = curM.getItem(ReportSumInt).isSelected();
				curEvent.reportNorm = curM.getItem(ReportNormInt).isSelected();
				curEvent.reportAve = curM.getItem(ReportAveInt).isSelected();
				curEvent.reportScaled = curM.getItem(ReportScaledInt).isSelected();*/
				curEvent.resetOuts();
				int curBIndex = dataOuts[i].getSelectedIndex();
				if (curBIndex == ReportSumInt) { curEvent.reportSum = true; }
				if (curBIndex == ReportNormInt) { curEvent.reportNorm = true; }
				if (curBIndex == ReportAveInt) { curEvent.reportAve = true; }
				if (curBIndex == ReportScaledInt) { curEvent.reportScaled = true; }
				
				eventSpecs[eventCt] = curEvent;
				eventCt++; 
			} else {
				onList[i] = false;
			}
		}
		if (specsFailed) { 
			talkln ("Failed to make event specs... check values");
			eventSpecs = null;
		}
		onList[0] = true;	// hack... load time column for sure
	}
	
	public boolean meetsPauseCriteria (Plateau curPlat) {
		// criteria for selecting plateaus; length of pause
		if ((curPlat.pauseTime > maxPauseTime) | (curPlat.pauseTime < minPauseTime)) { return false; }
		return true;
	}

	public boolean meetsPauseCriteria (Plateau lastPlat, Plateau curPlat, Plateau nextPlat, int sepSteps) {
		// criteria for selecting plateaus; length of pause
		if ((curPlat.pauseTime > maxPauseTime) | (curPlat.pauseTime < minPauseTime)) { return false; }
		// criteria for selecting plateaus; separation from adjacent pauses
		if ((curPlat.start - lastPlat.stop) < sepSteps) { return false; }
		if ((nextPlat.start - curPlat.stop) < sepSteps) { return false; } 
		return true;
	}
	
	public boolean meetsRunCriteria (Plateau curPlat, Plateau nextPlat, int profileRange) {
		if ((nextPlat.start - curPlat.stop) < profileRange) { return false; }
		return true;
	}
	
	public HistogramPlus [] makeEventProfiles() {
		talkln ("pause criteria of " + String.valueOf(minPauseTime) + " to " + String.valueOf(maxPauseTime));
		talkln ("separation criteria of " + String.valueOf(sepTime));
		int stepsMinus = 100;
		int stepsPlus = 100;
		int numBins = stepsMinus+stepsPlus;
		double deltaT = timeCol.data[1]-timeCol.data[0];
		double startTime = -1*stepsMinus*deltaT;
		double stopTime = stepsPlus*deltaT;
		int sepSteps = (int)(sepTime*1E-3/deltaT);	// # of steps required between adjacent pauses... sepTime in ms.
		Plateau lastPlat,curPlat,nextPlat;
		
		// make eventCols and eventHists	
		eventCols = new DataColumn[eventCt];		
		eventHists = new HistogramPlus[eventCt];
		for (int i=0;i<eventCt;i++) {
			eventCols[i] = inSet.dataTable[eventSpecs[i].index];
			eventHists[i] = new HistogramPlus(numBins,startTime,stopTime,"/",eventCols[i].name,this);
			eventHists[i].reportSum = eventSpecs[i].reportSum;
			eventHists[i].reportNorm = eventSpecs[i].reportNorm;
			eventHists[i].reportAve = eventSpecs[i].reportAve;
			eventHists[i].reportScaled = eventSpecs[i].reportScaled;
		}
		
		// set number of iterations... depends on type of alignment
		int numSteps = stepCol.lineCt;
		for (int i=1;i<numSteps-1;i++) {	// to the following for each plateau (except 1st and last) that passes criteria
			lastPlat = (Plateau)stepCol.lines[i-1];
			curPlat = (Plateau)stepCol.lines[i];
			nextPlat = (Plateau)stepCol.lines[i+1];
			// check if we want to use this plateau.... criteria is alignment dependents
			boolean usePlat;
			if (alignment == AlignMid_Run) { 
				usePlat = meetsRunCriteria(curPlat,nextPlat,numBins); 
			} else {
				usePlat = meetsPauseCriteria(lastPlat,curPlat,nextPlat,sepSteps);
			}
			if (usePlat) {
				int alignInd = -1;	// alignment index set below
				switch (alignment) {
					case AlignStart:
						alignInd = curPlat.start;
						break;
					case AlignMid:
						alignInd = (curPlat.start + curPlat.stop)/2;
						break;
					case AlignStop:
						alignInd = curPlat.stop;
						break;
					case AlignMid_Run:
						Plateau nextP = (Plateau)stepCol.lines[i+1];
						alignInd = (curPlat.stop + nextP.start)/2;
						break;
				}
				for (int k=0;k<eventCt;k++) { eventHists[k].addPass(); }
				for (int j=0;j<numBins;j++) {
					int curInd = j+alignInd-stepsMinus;
					if ((curInd >= 0) & (curInd < eventCols[0].rows)) {
						for (int k=0;k<eventCt;k++) {
							double newVal = 0;
							switch(eventSpecs[k].valType) {
								case TypeActual:
									newVal = eventCols[k].data[curInd]/eventSpecs[k].normFactor;
									break;
								case TypeRelative:
									newVal = (eventCols[k].data[curInd] - eventCols[k].data[alignInd])/eventSpecs[k].normFactor;
									break;
								case TypeAbsolute:
									newVal = Math.abs(eventCols[k].data[curInd]/eventSpecs[k].normFactor);
									break;
								case TypeAbsoluteRelative:
									newVal = Math.abs((eventCols[k].data[curInd] - eventCols[k].data[alignInd])/eventSpecs[k].normFactor);
									break;
							}
							eventHists[k].addValue(j, newVal);
						}
					}
				}	
			}
		}
		talkln ("used " + String.valueOf(eventHists[0].passCt) + " plateaus");
		return eventHists;
	}
	
	public HistogramPlus [] pauseCTethers (int startInd) {
		int bins = 1000;
		int tetMin = 0;
		int tetMax = 1000;
		
		DataColumn tetherCol = getColumnByName("#Tethers");
		//DataColumn tetherCol = getColumnByName("closeTips");
		
		HistogramPlus [] pct = new HistogramPlus [2];
		pct[0] = new HistogramPlus(bins,tetMin,tetMax,"/",tetherCol.name + "-Pause Correlation",this);
		pct[1] = new HistogramPlus(bins,tetMin,tetMax,"/",tetherCol.name + "-Distribution",this);
		pct[0].reportNorm = true;
		pct[1].reportNorm = true;
		
		Plateau lastPlat,curPlat,nextPlat;
		// load tether numbers during pauses
		for (int i=1;i<stepCol.lineCt-1;i++) {
			lastPlat = (Plateau)stepCol.lines[i-1];
			curPlat = (Plateau)stepCol.lines[i];
			nextPlat = (Plateau)stepCol.lines[i+1];
			if (meetsPauseCriteria(curPlat)) {
				pct[0].addPass();
				for (int j=curPlat.start;j<curPlat.stop;j++) {
					pct[0].addValue(tetherCol.data[j]);
				}
			}
		}
				
		// load all tether numbers
		for (int i=startInd;i<timeCol.rows;i++) {
			pct[1].addValue(tetherCol.data[i]);
		}
		
		return pct;
	}
	
	
	public DataColumn getColumnByName (String name) {
		for (int i=0;i<inSet.dims[0];i++) {
			String curName = inSet.dataTable[i].name;
			if (curName.equals(name)) {
				return inSet.dataTable[i];
			}
		}
		return null;
	}
	
	
	public void actionPerformed (ActionEvent e) {
		String act = e.getActionCommand();
	
		if (act.equals("Work")) {
			parseGUI();
			HistogramPlus [] hists = makeEventProfiles();
			FileOps.mkHistogramFile (hists);
		}
		
		if (act.equals("EventSelected")) {
			setEnabledFields();
		}
		
	}
	
	public void keyReleased (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
	}
	public void keyTyped (KeyEvent e) {}
	public void keyPressed (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
		
		if (act.equals("E")) {
			makeEventSpecs();
			HistogramPlus [] hists = makeEventProfiles();
			FileOps.mkHistogramFile (hists);
		}
	}
	
	public void stateChanged(ChangeEvent e) {   	
        		 
    } 
    
    public void makeControlPanel() {
    	//work button
		workB = new JButton("Work");
		workB.setActionCommand ("Work");
		workB.setToolTipText ("Event histogram with current settings");
		//workB.setFont(Env.controlFont);
		workB.addActionListener(this);
		
		// alignment selection
		alignType.setToolTipText ("Align events by this location");
		
		// selection criteria
		JLabel maxPauseLabel = new JLabel ("max pause (ms)");
		JLabel minPauseLabel = new JLabel ("min pause (ms)");
		maxPause = new JTextField("10");
		minPause = new JTextField("0");
		JPanel maxPP = new JPanel(new GridLayout(1,2));
		JPanel minPP = new JPanel(new GridLayout(1,2));
		maxPP.add(maxPauseLabel);
		maxPP.add(maxPause);
		minPP.add(minPauseLabel);
		minPP.add(minPause);
		
		controlPanel.setLayout(new GridLayout(2,2));
		controlPanel.add(workB);
		controlPanel.add(alignType);
		controlPanel.add(minPP);
		controlPanel.add(maxPP);
	}
	
	private void talk (String info) {
		System.out.print(info);
	}
	
	private void talkln (String info) {
		System.out.println(info);
	}

	
}