package rocketBugsRender;
/* 
	ZoomPanel.... the class defining a window to show finer detail
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.awt.image.BufferedImage;
import javax.swing.*;
import java.text.*;
import java.io.*;
import org.jibble.epsgraphics.*;

public class RBRPanel extends JPanel {
	RocketBugsRender MP;		// a reference back to the parent JFrame of this panel
	static BufferedImage theImage;
	static Graphics2D thePalate;
	static EpsGraphics2D epsG;
	static int winSize;
	double scale,windowfact,winScale;
	double halfWindow;		// dimension of half the zoom window width
	double dispAng = 0;
	double [] trigDispAng = {0,1};  	// store {sin(dispAng), cos(dispAng)}
	Pt3D offset = new Pt3D ();	// store {x,y,z} offsets for the zoom window
	Pt3D zoomCen = new Pt3D ();	// store zoom center
	int [] cursorPos = {0,0};	// store initial cursor position for dispAng and Z changes
	PinCam zoomCam;
	DecimalFormat scaleBarFormat = new DecimalFormat ("#0.0#; -#0.0#");
	
	// BasicStroke definitions for drawing
	BasicStroke normStroke = new BasicStroke (1.0f,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND);
	BasicStroke wideStroke = new BasicStroke (2.0f,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND);
	
	// for eps out
	static boolean epsOut = false;
	
	public RBRPanel (int winSize, Pt3D cenPt, double scale, RocketBugsRender MP) {
		this.MP = MP;
		this.winSize = winSize;
		this.scale = scale;
		windowfact = winSize/(Env.dimensions.x*2);
		winScale = scale*windowfact;
		halfWindow = winSize/(2*winScale);
		Pt3D distantViewPoint = new Pt3D(Env.dimensions.x/2, Env.dimensions.y/2, -100);
		zoomCam = new PinCam (zoomCen, offset, new Pt3D (0,0,1), .8);
		zoomCam.newPixelLimits(new Pixel (winSize, winSize));
		newZoomCenter(cenPt);
		setSize(winSize,winSize);
	}
	
	public void initialize() {
		theImage = (BufferedImage)createImage(winSize,winSize);
		setThePalateForScreen();
	}
	
	private void setThePalateForScreen () {
		thePalate = (Graphics2D)theImage.getGraphics();
		thePalate.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
		thePalate.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
		thePalate.setStroke(normStroke);
	}
	
	public void openEpsFile () {
	
		try {
			String epsFileName = FileOps.getFileToSave("Save EPS file as...", MP);
			File epsFile = new File(epsFileName);
			String title = "EPS grab";
			epsG = new EpsGraphics2D(title,epsFile,0,0,winSize,winSize);
		} catch (IOException ioe) { talkln ("Problem creating EpsGraphics2D in ZoomPanel.java"); }
		thePalate = epsG;	// switch reference to eps file
	}
	
	public void closeEpsFile () {
		try {
			epsG.close();
		} catch (IOException ioe) { talkln ("Problem closing epsG in PlotPanel.java"); }
		epsOut = false;
		setThePalateForScreen();
	}
	
	public void newZoomCenter (Pt3D zoomCen) {
		this.zoomCen = zoomCen;
		offsetsFromZoomCen ();
		updatePinCam ();
	}
	
	public void newZoomCenter (double [] cenPt) {
		zoomCen.x = cenPt[0];
		zoomCen.y = cenPt[1];
		zoomCen.z = cenPt[2];
		offsetsFromZoomCen ();
		updatePinCam ();
	}
	
	public void offsetsFromZoomCen () {
		offset.y = zoomCen.y - halfWindow;
		offset.x = zoomCen.x - halfWindow*trigDispAng[1];
		offset.z = zoomCen.z + halfWindow*trigDispAng[0];
	}
	
	public void newScale (double newS) {
		scale = newS;
		winScale = scale*windowfact;
		halfWindow = winSize/(2*winScale);
		offsetsFromZoomCen();
		updatePinCam ();
		//talkln ("new scale is " + String.valueOf(scale));
		RocketBugsRender.setScaleField();
	}
	
	public void newDispAng (double newAng) {
		dispAng = newAng;
		if (dispAng > 2*Math.PI) { dispAng += -2*Math.PI; }
		if (dispAng < -2*Math.PI) { dispAng += 2*Math.PI; }
		trigDispAng[0] = Math.sin(dispAng);
		trigDispAng[1] = Math.cos(dispAng);
		offsetsFromZoomCen();
		updatePinCam ();
	}
	
	public void incDispAng (double angInc) {
		dispAng += angInc;
		newDispAng(dispAng);
	}
	
	public void chgZValue (double deltaZ) {
		zoomCen.x += deltaZ*trigDispAng[0];
		zoomCen.z += deltaZ*trigDispAng[1];
		offsetsFromZoomCen();
		updatePinCam ();
	}
	
	public void chgYValue (double deltaY) {
		zoomCen.y += deltaY;
		offsetsFromZoomCen();
		updatePinCam();
	}
	
	public void setCursorPos (int x, int y) {
		cursorPos[0] = x;
		cursorPos[1] = y;
	}	
	
	public void chgZoomAng (int x, int y) {
		double newAng = dispAng + (Math.PI/2)*(cursorPos[0]-x)/(winSize/2);
		newDispAng (newAng);
		MP.updateInfo();	
	}
	
	public void chgZoomZ (int x, int y) {
		double deltaZ = Env.dimensions.z*(cursorPos[1]-y)/(winSize);
		chgZValue (deltaZ);
		MP.updateInfo();	
	}
	
	public void chgZoomY (int x, int y) {
		double deltaY = (cursorPos[1]-y)/(winScale);
		chgYValue(deltaY);
		MP.updateInfo();
	}
	
	public void updatePinCam () {
		zoomCam.changePos (zoomCen, offset);
		zoomCam.changeDir(dispAng);
	}
	
	public void update(Graphics g) {
		paint(g);
	}
	
	public void paint (Graphics g) {
		g.drawImage(theImage,0,0,null);
	}
	
	public Pt3D convertPixelToPoint (int xPix, int yPix) {
		Pt3D point = new Pt3D();
		point.x = offset.x + trigDispAng[1]*(double)(xPix)/winScale;
		point.y = offset.y + (double)(yPix)/winScale;
		point.z = offset.z - trigDispAng[0]*(double)(xPix)/winScale;
		return point;
	}

	public void clearImage() {
		thePalate.setColor(Env.backgroundColor);
		thePalate.fillRect(0,0,winSize,winSize);
		thePalate.setColor(Env.borderColor);
		thePalate.drawRect(0,0,winSize-1,winSize-1);
	}
	
	public void drawPlayer (Player player) {
		player.showYourself(thePalate, winScale, zoomCam);
	}
	
	public void drawPlayerList (Player [] playerList, int playerCount) {
		for (int i = 0; i < playerCount; i++) {
			playerList[i].showYourself(thePalate, winScale, zoomCam);
		}
	}
	
	public void showPath () {
		thePalate.setColor(Env.trackPathColor);
		Pixel pix1 = new Pixel();
		Pixel pix2 = new Pixel();
		if (Env.connectPts) {
			pix1.ptToPix(Env.pathPts[0],winScale,zoomCam.offset,zoomCam.e);
			for (int i=1;i<Env.pathPtCt;i++) {
				pix2.ptToPix(Env.pathPts[i],winScale,zoomCam.offset,zoomCam.e);
				Pixel.drawLine(thePalate, pix1, pix2);
				pix1.copy(pix2);
			}
		} else {
			int pixRad = Env.pathCircleRad;
			for (int i=0;i<Env.pathPtCt;i++) {
				pix1.ptToPix(Env.pathPts[i],winScale,zoomCam.offset,zoomCam.e);
				thePalate.drawOval(pix1.x-pixRad,pix1.y-pixRad,2*pixRad,2*pixRad);
			}
		}
	}
	
	public void showScaleBar () {
		thePalate.setColor(Env.scaleBarColor);
		thePalate.setFont(Env.scaleBarFont);
		int endBarSize = 3;
		Pixel pixA = new Pixel(20,20);
		Pixel pixB = new Pixel(pixA.x + (int)(Env.scaleBarLength*winScale), pixA.y);
		Pixel.drawLine(thePalate, pixA, pixB);
		Pixel barA = new Pixel(pixA.x, pixA.y-endBarSize);
		Pixel barB = new Pixel(pixA.x, pixA.y+endBarSize);
		Pixel.drawLine(thePalate, barA, barB);
		barA = new Pixel(pixB.x, pixB.y-endBarSize);
		barB = new Pixel(pixB.x, pixB.y+endBarSize);
		Pixel.drawLine(thePalate, barA, barB);
		// text for scale bar
		String scaleString;
		if (Env.scaleBarLength >= 1.0) {
			scaleString = String.valueOf(scaleBarFormat.format(Env.scaleBarLength)) + " m";	// scale bar string in micrometers
		} else {
			scaleString = String.valueOf(scaleBarFormat.format(Env.scaleBarLength*1000)) + " nm";	// scale bar string in nanometers
		}
		thePalate.drawString(scaleString, (int)((pixA.x + pixB.x)/2-12), pixA.y-2*endBarSize);
	}
	
	public void showTime () {
		thePalate.setColor(Env.timeColor);
		thePalate.setFont(Env.timeFont);
		String timeString = String.valueOf(Env.timeFormat.format(Env.curTime)) + " sec";
		Pixel pixA = new Pixel(Env.zoomWindowSize - 75, 14);
		thePalate.drawString(timeString, pixA.x, pixA.y);
	}
	
	private void talk (String info) {
		System.out.print(info);
	}
	
	private void talkln (String info) {
		System.out.println(info);
	}
}