package rocketBugsRender;
/*
	RocketBugsRender.... a jframe to play movies
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import infoCCD.Info;

import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.text.*;

public class RocketBugsRender extends JFrame implements ActionListener, WindowListener, ComponentListener, KeyListener, ChangeListener, MouseListener, MouseMotionListener {
	FileOps fileOps;
	static RBRPanel zoomPanel;
	JPanel infoBox = new JPanel();
	JPanel controlBox = new JPanel();
	JPanel playBox = new JPanel();
	JPanel sliderBox = new JPanel();
	JPanel southPanel = new JPanel();
	JPanel northPanel = new JPanel();
	JPopupMenu analysisMenu;
	JLabel moviePath = new JLabel("No Movie Loaded... Press 'Play'");
	JLabel viewDist, focalLength, eyeSep, eyeAng;
	GridBagLayout southLay;		// layout manager for the southPanel
	MovieThread playThread;
	final JFileChooser jfc;
	File mvName = null;
	File toRawMv;
	File toPNGMv;
	String filePath;
	String fileType;
	String [] mvFileNames;
	String baseName;
	JSlider magSlider, focalSlider, viewDistSlider, eyeSepSlider, eyeAngSlider;
	JLabel displayAngle, zoomCenter, unitVector;
	static JTextField scaleField;
	JButton rotateButton, playButton, stopButton, eyesButton;
	JToggleButton paintButton, movieButton, trackActinB;
	JComboBox actinOptionsBox;
	Icon rotateIcon, noRotateIcon, playIcon, pauseIcon, stopIcon, stoppedIcon, makeMovieIcon, makingMovieIcon;
	Icon paintIcon, noPaintIcon, eyesIcon, eyesOnIcon, viewIcon;
	Cursor crossHair = new Cursor(Cursor.CROSSHAIR_CURSOR);
	boolean paused = true;
	DecimalFormat pngIdFormat = new DecimalFormat ("#0000.#;#0000.#");
	DecimalFormat pausedIdFormat = new DecimalFormat ("#0000.#;#0000.#");
	static DecimalFormat scaleReportFormat = new DecimalFormat ("#0.00#; #0.00#");
	int pausedId = 0;
	int viewId = 0;
	int pngId = 0;
	
	int frame = 0;			// counter to keep track of frame to draw
	int numOfFrames = 0; 	// number of frames to show
	int magSliderSpace = 20;
	int infoBoxSpace = 20;
	boolean chgAng = false;
	boolean chgZ = false;
	boolean chgY = false;
	boolean firstDrag = false;
	DecimalFormat coordOut = new DecimalFormat ("#0.00#; -#0.00#");
	int [] gridPt = new int [3];	// test stuff
	
	// some divisors to give appropriate values from integer sliders
	double focalDiv = 100;
	double eyeSepDiv = 100;
	double viewDistDiv = 10.0;
	double eyeAngDiv = 10.0;
	
	// listeria info plotting and analysis
	boolean lisInfoLoaded = false;
	String [] lisInfoColNames;
	
	public RocketBugsRender (String [] inputParams) {
		if (inputParams.length != 0) { parseParams(inputParams); }
		
		// make InfoCCD materials
		new Info(true,true,false);
		makeMenuBar();
		
		// make instance of FileOps class to reference and instantiate JFileChooser
		jfc = new JFileChooser ();
		fileOps = new FileOps();
		
		Pt3D cenPt = new Pt3D (10,10,0);
		zoomPanel = new RBRPanel(Env.zoomWindowSize, cenPt, Env.initialScale, this);
		makeMagSlider ();
		makeInfoBox ();
		makeControlBox();
		makeSliderBox();
		makeSouthPanel();
		makePlayBox();
		makeNorthPanel();
		
		setWindowSize ();
		setResizable(true);
		setLocation(0,0);
		initialize();
		setCursor(crossHair);

		makePopUp();
		
		addKeyListener(this);
		addWindowListener(this);
		addComponentListener(this);
		addMouseListener(this);
		addMouseMotionListener(this);
		
		if (Env.movieName != null) { setMovieToPlay(Env.movieName, Env.fileType); }
			
	}
	
	public void parseParams (String [] inputParams) {
		for (int i = 0; i < inputParams.length; i++) {
			if (inputParams[i].equals("-help") | inputParams[i].equals("?")) {
				System.out.println ("The following command line arguments are accepted.....");
				System.out.println ("	-help	 prints this help message");
				System.out.println ("	-is [double] start with this scale factor for drawing");
				System.out.println ("	-zc [double,double,double] start with the drawing centered at this x,y,z");
				System.out.println ("	-qk [directory name] the .qk movie files are here");
				System.out.println (" 	-sk [integer] how many files to skip between frames");
				System.exit(0);
			}
			if (inputParams[i].equals("-is"))  {
				Double dd = Double.valueOf(inputParams[i+1]);
				Env.initialScale = dd.doubleValue();
			}
			
			if (inputParams[i].equals("-sk")) {
				Double ii = Double.valueOf(inputParams[i+1]);
				Env.framesToSkip = (int) (ii.doubleValue());
			}
			
			if (inputParams[i].equals("-qk")) {
				// this filename assembly is "space" friendly.. good for Mac and "Desktop Folder"
				String assembleName = inputParams[i+1];
				int j = 2;
				while (i+j < inputParams.length) {
					if (inputParams[i+j].startsWith("-")) {
						j = 1000; // exit
					} else {
						assembleName = assembleName + " " + inputParams[i+j];
						j++;	  // increment
					}
				}
				File mvdir = new File (assembleName);
				if (mvdir.isDirectory()) {
					Env.movieName = mvdir;
					Env.fileType = "qk";
				} else {
					talkln ("Sorry, can't find " + mvdir.getAbsolutePath() + "  .... try again.");
				}
			}
		}
	}
	
	public void initialize () {
		setVisible(true);
		getContentPane().add(northPanel,"North");
		getContentPane().add(southPanel,"South");
		setVisible(true);
	}
	
	public class MovieThread extends Thread {
		public MovieThread() {
			this.setDaemon(true);
			paused = true;
			this.start();
		}
		
		public void run () {
			frame = 0;
			numOfFrames = mvFileNames.length;
			fileOps.paintQkFrameFromFile (filePath + "/" + mvFileNames[0], zoomPanel, true);
			zoomPanel.newZoomCenter(Env.curLisPos);	// set center on end1 of Listeria
			while (true) {
					if (!paused) {
					frame += Env.framesToSkip;
					if (frame > numOfFrames-1) { frame = numOfFrames-1; }
				}
				moviePath.setText(mvFileNames[frame] + "   " + String.valueOf(frame+1) + "/" + String.valueOf(numOfFrames));
				zoomPanel.clearImage();
				Env.calculateCurrentTime(mvFileNames[frame]);		// calculate time from file name
				// eps out stuff
				if (RBRPanel.epsOut) {
					zoomPanel.openEpsFile();
					fileOps.paintQkFrameFromFile (filePath + "/" + mvFileNames[frame], zoomPanel, false);
					zoomPanel.closeEpsFile();
				}
				// end eps out stuff
				if (!paused) {
					Env.paused = false;
					fileOps.paintQkFrameFromFile (filePath + "/" + mvFileNames[frame], zoomPanel, false);
				} else {
					Env.paused = true;
					if (! Env.curFileLoaded) { fileOps.loadQkFrameFromFile (filePath + "/" + mvFileNames[frame], zoomPanel, false); }
					zoomPanel.drawPlayerList(Player.playerList, Player.playerCount);
				}
				if ((frame == 0) & (paused) & (! Env.trackActin) & (! Env.makePNGMovie) & (! Env.trackListeria)) {	// for intro paint listeria position at halfway and end of movie frames
					int halfwayFrame = (int)(numOfFrames/2);
					fileOps.paintQkFrameFromFile (filePath + "/" + mvFileNames[halfwayFrame], zoomPanel, true);
					fileOps.paintQkFrameFromFile (filePath + "/" + mvFileNames[numOfFrames-1], zoomPanel, true);
				}
				if (Env.drawScaleBar) { zoomPanel.showScaleBar(); }
				if (Env.drawTime) { zoomPanel.showTime(); }
				if (Env.paintScreen) { zoomPanel.repaint(); }
				try { Thread.sleep(5); } catch (Exception e) {talkln ("Some sort of MovieThread sleep problem"); }
				if (Env.makePNGMovie) { takePNGPic(); }
				if (Env.rotateZoomAngle) { rotateZoomAngle(); }
				if (Env.autoZoomIn) { zoomPanel.newScale(zoomPanel.scale + .01*zoomPanel.scale); }
				if (Env.autoZoomOut) { zoomPanel.newScale(zoomPanel.scale - .01*zoomPanel.scale); }
				if (frame == numOfFrames-1) {
					if (Env.rotateAtEnd) { turnTurnTurn(); } 
					stopMovie(); 
				}	// if we reach end of movie then pause
			}
		}
	}
	
	public void turnTurnTurn() {
		paused = true;	// only set this for filename purposes in takeRawPic()
		fileOps.loadQkFrameFromFile (filePath + "/" + mvFileNames[frame], zoomPanel, false);
		for (int i=0; i<180; i++) {
			zoomPanel.clearImage();
			zoomPanel.drawPlayerList(Player.playerList, Player.playerCount);
			if (Env.drawScaleBar) { zoomPanel.showScaleBar(); }
			if (Env.drawTime) { zoomPanel.showTime(); }
			if (Env.trackPath) { zoomPanel.showPath(); }
			if (Env.paintScreen) { zoomPanel.repaint(); }
			//try { this.sleep(5); } catch (Exception e) {talkln ("Some sort of MovieThread sleep problem"); }
			if (Env.makePNGMovie) { takePNGPic(); }
			rotateZoomAngle();
		}
	}
	
	public void updateInfo () {
		displayAngle.setText("  X-Z viewing angle: " + String.valueOf(coordOut.format(zoomPanel.dispAng*180/Math.PI)));
		String [] coordStr = new String [3];
		String [] uStr = new String [3];
		uStr[0] = String.valueOf(coordOut.format(zoomPanel.zoomCam.e.x));
		uStr[1] = String.valueOf(coordOut.format(zoomPanel.zoomCam.e.y));
		uStr[2] = String.valueOf(coordOut.format(zoomPanel.zoomCam.e.z));
		coordStr[0] = String.valueOf(coordOut.format(zoomPanel.zoomCen.x));
		coordStr[1] = String.valueOf(coordOut.format(zoomPanel.zoomCen.y));
		coordStr[2] = String.valueOf(coordOut.format(zoomPanel.zoomCen.z));
		unitVector.setText("  Unit Vector: (" + uStr[0] + "," + uStr[1] + "," + uStr[2] + ")");
		zoomCenter.setText("  Coordinates: (" + coordStr[0] + "," + coordStr[1] + "," + coordStr[2] + ")");
		if (Env.usePinCam) {
			viewDist.setText("  View Distance: " + String.valueOf(viewDistSlider.getValue()/viewDistDiv));
			focalLength.setText("  Focal Length: " + String.valueOf(focalSlider.getValue()/focalDiv));
			eyeSep.setText("  Eye Separation: " + String.valueOf(eyeSepSlider.getValue()/eyeSepDiv));
			eyeAng.setText("  Eye Angle: " + String.valueOf(eyeAngSlider.getValue()/eyeAngDiv) + " ");
		}
	}
	
	public void newZoomCenter (Pt3D point) {
		zoomPanel.newZoomCenter (point);
		updateInfo ();
	}
	
	public void resetZoomAngle () {
		zoomPanel.newDispAng(0);
		updateInfo ();
	}
	
	public void rotateZoomAngle () {
		rotateZoomAngle (Env.viewAngleStep);
	}
	
	public void rotateZoomAngle (double angInc) {
		zoomPanel.incDispAng(angInc);
		updateInfo();
	}
	
	public void getMovieToPlay () {
		//new File_Name_Getter(this);
		jfc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int fileQuery = jfc.showOpenDialog(this);
		if (fileQuery == JFileChooser.APPROVE_OPTION) {
            File file = jfc.getSelectedFile();
            setMovieToPlay (file, "qk");
            talkln ("Opening: " + file.getName());
        } else {
            talkln ("Open command cancelled by user");
        }
	}
	
	public void setMovieToPlay (File mvName, String fileType) {
		this.mvName = mvName;
		this.baseName = mvName.getName();
		this.fileType = fileType;
		filePath = mvName.getAbsolutePath();
		String [] tempFileList = mvName.list();
		// my own little filter for .qk files
		int qkCount = 0;
		for (int i = 0; i < tempFileList.length; i++) {
			if (tempFileList[i].endsWith(".qk")) {
				qkCount++;
			} else {
				tempFileList[i] = null;
			}
		}
		mvFileNames = new String [qkCount];
		int addCount = 0;
		for (int i = 0; i < tempFileList.length; i++) {
			if  (tempFileList[i] != null) {
				mvFileNames[addCount] = tempFileList[i];
				addCount++;
			}
		}
		setTitle (baseName + "." +  fileType);
		if (Env.makePNGMovie) { makePNGMovieDir (); }
		getContentPane().add(zoomPanel,"Center");
		zoomPanel.initialize();
		getContentPane().add(magSlider,"East");
		setWindowSize();
		this.setVisible(true);
		playThread = new MovieThread();
	}
	
	public void loadListeriaInfo() {
		if (!lisInfoLoaded) {
			String lisInfo = filePath + "/" + baseName + ".DATA/" + baseName + "ListeriaInfo";
			new DataSet (lisInfo);
		}
	}
	
	public void makeRawMovieDir () {
		toRawMv = new File (mvName.getAbsolutePath() + ".RAW");
		int j=1;
		while (toRawMv.isDirectory()) {
			toRawMv = new File(mvName.getAbsolutePath() + ".RAW" + "." + String.valueOf(j));
			j++;
		}
		toRawMv.mkdir();
	}
	
	public void makePNGMovieDir () {
		toPNGMv = new File (mvName.getAbsolutePath() + ".PNGS");
		int j=1;
		while (toPNGMv.isDirectory()) {
			toPNGMv = new File(mvName.getAbsolutePath() + ".PNGS" + "." + String.valueOf(j));
			j++;
		}
		toPNGMv.mkdir();
	}
	
	public void takeRawPic () {
		String filename;
		String wholeString = mvFileNames[frame];
		String baseName = wholeString.substring(0, wholeString.length()-3);
		if (paused) {
			String addOn = String.valueOf(pausedIdFormat.format(pausedId));
			filename = toRawMv.getAbsolutePath() + "/" + baseName + addOn + ".raw";
			pausedId++;
		} else {
			pausedId = 0;		// reset paused file id counter
			filename = toRawMv.getAbsolutePath() + "/" + baseName + "0000" + ".raw";
		}
		FileOps.takeRawPicture (RBRPanel.theImage,filename);
	}
	
	public void takePNGPic () {
		String filename;
		String baseName = "file";
		if (paused) {
			pausedId++;
			String pngNum = String.valueOf(pngIdFormat.format(pngId));
			String addOn = String.valueOf(pausedIdFormat.format(pausedId));
			filename = toPNGMv.getAbsolutePath() + "/" + baseName + pngNum + addOn + ".png";
		} else {
			pngId++;
			pausedId = 0;		// reset paused file id counter
			String pngNum = String.valueOf(pngIdFormat.format(pngId));
			String addOn = String.valueOf(pausedIdFormat.format(0));
			filename = toPNGMv.getAbsolutePath() + "/" + baseName + pngNum + addOn + ".png";
		}
		FileOps.pngFromBufferedImage(RBRPanel.theImage,filename);
	}
	
	public void playMovie () {
		paused = false;
		Env.curFileLoaded = false;
		if (frame == numOfFrames-1) { frame = 0; }		// allow play button to start movie over again
		//stopButton.setIcon(stopIcon);
		//playButton.setIcon(pauseIcon);
		playButton.setEnabled(true);
		playButton.setActionCommand("Pause");
	}
	
	public void pauseMovie () {
		paused = true;
		//playButton.setIcon(playIcon);
		playButton.setEnabled(false);
		playButton.setActionCommand("Play");
	}
	
	public void stopMovie () {
		stopButton.setIcon(stoppedIcon);
		pauseMovie();
		playButton.setIcon(playIcon);
		playButton.setActionCommand("Play");
		Env.makePNGMovie = false;
	}
	
	public void setWindowSize() {
		if (mvName == null) {
			setSize(560,118);
			//setSize (southPanel.getPreferredSize().width, southPanel.getPreferredSize().height+northPanel.getPreferredSize().height+20);
		} else {
			setSize (Env.zoomWindowSize + magSliderSpace, Env.zoomWindowSize + southPanel.getPreferredSize().height);
		}
	}

	public static void setScaleField () {
		String scaleStr = String.valueOf(scaleReportFormat.format(zoomPanel.scale));
		scaleField.setText(scaleStr);
	}
	
	public void actionPerformed (ActionEvent e) {
		String act = e.getActionCommand();
		if (act.equals("Play")) {
			if (mvName != null) { playMovie();
			} else { getMovieToPlay(); }
		}
		
		if (mvName != null) {		// all these other buttons are active only if a movie is loaded
			if (act.equals("Pause")) { pauseMovie(); }
			if (act.equals("StopPlay")) { stopMovie(); }
			if (act.equals("NextFrame")) { if (frame < numOfFrames-1) { frame++; }}
			if (act.equals("PreviousFrame")) { if (frame > 0) { frame += -1; }}
		
			if (act.equals("rotate")) {
				if (Env.rotateZoomAngle == true) {
					Env.rotateZoomAngle = false;
					rotateButton.setSelected(false);
				} else {
					//resetZoomAngle();
					Env.rotateZoomAngle = true;
					rotateButton.setSelected(true);
				}
			}
			/*if (act.equals("makeMovie")) {
				if (Env.makeRawMovie == true) {
					Env.makeRawMovie = false;
					movieButton.setSelected(false);
				} else {
					Env.makeRawMovie = true;
					movieButton.setSelected(true);
					setResizable(false);
					makeRawMovieDir();
				}
			}*/
			
			if (act.equals("makeMovie")) {
				if (Env.makePNGMovie == true) {
					Env.makePNGMovie = false;
					movieButton.setSelected(false);
				} else {
					Env.makePNGMovie = true;
					movieButton.setSelected(true);
					setResizable(false);
					makePNGMovieDir();
				}
			}
		
			if (act.equals("PinHole View")) {
				if (Env.usePinCam == true) {
					Env.usePinCam = false;
					eyesButton.setIcon(eyesIcon);
					southPanel.remove(sliderBox);
					setWindowSize();
					this.setVisible(true);
				} else {
					Env.usePinCam = true;
					eyesButton.setIcon(eyesOnIcon);
					GridBagConstraints southLayCons = new GridBagConstraints();
					southLayCons.ipadx = 0;
					southLayCons.ipady = 5;
					southLayCons.gridx = 0;
					southLayCons.gridy = 1;
					southLay.setConstraints(sliderBox, southLayCons);
					southPanel.add(sliderBox);
					setWindowSize();
					updateInfo();
					this.setVisible(true);
				}
			}
			
			if (act.equals("trackActin")) {
				if (Env.trackActin) { 
					Env.trackActin = false;
					trackActinB.setSelected(false);
					ActinTracker.toFile();
				} else {
					Env.trackActin = true;
					trackActinB.setSelected(true);
				}
			}
			
			if (act.equals("actinOptions")) {
				int actinView = actinOptionsBox.getSelectedIndex();
				Env.drawActinHelix = false;
				Env.drawEachMonomer = false;
				Env.drawHelicalLines = false;
				if (actinView == 0) {
					Env.drawActinHelix = true;
					Env.drawEachMonomer = true;
					Env.drawHelicalLines = true;
				}
				if (actinView == 1) {
					Env.drawActinHelix = true;
					Env.drawEachMonomer = true;
				}
				if (actinView == 2) {
					Env.drawActinHelix = true;
					Env.drawHelicalLines = true;
				}
			}					
			
			
			if (act.equals("paintOff")) {
				Env.paintScreen = false;
				paintButton.setSelected(false);
				paintButton.setActionCommand ("paintOn");
				paintButton.setToolTipText ("Show frames on screen");
			}
			
			if (act.equals("paintOn")) {
				Env.paintScreen = true;
				paintButton.setSelected(true);
				paintButton.setActionCommand("paintOff");
				paintButton.setToolTipText ("Don't show frames on screen");
			}
		}
		
		if (act.equals("manualScaleSet")) {
			try {
				Double valD = Double.valueOf(scaleField.getText());
				double val = valD.doubleValue();
				zoomPanel.newScale(val);
			} catch (java.lang.NumberFormatException nfe) {
				System.out.println("Invalid field entry in setting scale... the change was ignored");
			}
		}
		
		if (act.equals("zoomIn")) { 
			Env.autoZoomOut=false;
			Env.autoZoomIn = !Env.autoZoomIn;
			//if (Env.autoZoomIn) { Env.autoZoomIn = false; } else { Env.autoZoomIn = true; Env.autoZoomOut = false;}
		}
		
		if (act.equals("zoomOut")) { 
			Env.autoZoomIn = false;
			Env.autoZoomOut = !Env.autoZoomOut;
			//if (Env.autoZoomOut) { Env.autoZoomOut = false; } else { Env.autoZoomOut = true; Env.autoZoomIn = false;}
		}
		
		if (act.equals("skipPlus")) {
			Env.framesToSkip ++;
			talkln (String.valueOf(Env.framesToSkip) + " files between frames");
		}
		if (act.equals("skipMinus")) {
			Env.framesToSkip += -1;
			if (Env.framesToSkip < 1) { Env.framesToSkip = 1; }
			talkln (String.valueOf(Env.framesToSkip) + " files between frames");
		}
		
		
		// pop up analysis options
		if (act.equals("Plot")) {
			new PlotFrame(this);
			//talkln("Plotting variables");
			//loadListeriaInfo();
		}
		
		if (act.equals("Batch Histograms")) {
			new BatchHist();
		}
		
		if (act.equals("Pairwise Histogram")) {
			new PairWiseDisp();
		}
		
		if (act.equals("epsOut")) {
			RBRPanel.epsOut = true;
		}
	}
	
	public void stateChanged (ChangeEvent e) {
		zoomPanel.newScale ((double)(magSlider.getValue())/100);
		zoomPanel.zoomCam.changeFocalLength(focalSlider.getValue());
		updateInfo();
	}
	
	public void mouseClicked (MouseEvent e) {
		Pt3D point = zoomPanel.convertPixelToPoint(e.getX(),e.getY());
		newZoomCenter(point);
	}
	public void mouseEntered (MouseEvent e) {}
	public void mouseExited (MouseEvent e) {}
	public void mousePressed (MouseEvent e) {
		if (e.isPopupTrigger()) {
			analysisMenu.show(this,e.getX(),e.getY());
		}
	}
	public void mouseReleased (MouseEvent e) {}
	public void mouseDragged (MouseEvent e) {
		if (chgAng) {
			if (firstDrag) {
				zoomPanel.setCursorPos (e.getX(), e.getY());
				firstDrag = false;
			}
			zoomPanel.chgZoomAng (e.getX(), e.getY());
			zoomPanel.setCursorPos (e.getX(), e.getY());
		}
		if (chgZ) {
			if (firstDrag) {
				zoomPanel.setCursorPos (e.getX(), e.getY());
				firstDrag = false;
			}
			zoomPanel.chgZoomZ (e.getX(), e.getY());
			zoomPanel.setCursorPos (e.getX(), e.getY());
		}
		if (chgY) {
			if (firstDrag) {
				zoomPanel.setCursorPos (e.getX(), e.getY());
				firstDrag = false;
			}
			zoomPanel.chgZoomY (e.getX(), e.getY());
			zoomPanel.setCursorPos (e.getX(), e.getY());
		}
	}
	public void mouseMoved (MouseEvent e) {}
	
	public void windowActivated (WindowEvent e) {}
	public void windowClosing (WindowEvent e) { System.exit(0); }
	public void windowDeactivated (WindowEvent e) {}
	public void windowDeiconified (WindowEvent e) {}
	public void windowIconified (WindowEvent e) {}
	public void windowOpened (WindowEvent e) {}
	public void windowClosed (WindowEvent e) {}
	
	public void componentHidden (ComponentEvent e) {}
	public void componentShown (ComponentEvent e) {}
	public void componentMoved (ComponentEvent e) {}
	public void componentResized (ComponentEvent e) {
		//playThread.suspend();
		Dimension newSize = this.getSize();
		talkln ("newSize is " + String.valueOf(newSize.width) + "," + String.valueOf(newSize.height));
		/*
		Env.zoomWindowSize = this.getSize().width - magSliderSpace;
		zoomPanel = new ZoomPanel(Env.zoomWindowSize, zoomPanel.zoomCen, zoomPanel.scale, this);
		getContentPane().add(zoomPanel, "Center");
		zoomPanel.initialize();
		setWindowSize();
		talkln ("New Window Size is " + String.valueOf(Env.zoomWindowSize));
		paintComponents(getGraphics());
		//playThread.resume();
		*/
	}
	
	public void keyReleased (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
		if (act.equals("A")) { chgAng = false; }
		if (act.equals("Z")) { chgZ = false; }
		if (act.equals("Q")) { chgY = false; }
	}
	public void keyTyped (KeyEvent e) {}
	public void keyPressed (KeyEvent e) {
		char key = e.getKeyChar();
		if (key =='A') { chgAng = true; firstDrag = true; }
		if (key == 'Z') { talkln("zPressed"); chgZ = true; firstDrag = true; }
		if (key == 'Q') { chgY = true; firstDrag = true; }
		
		if (key == 'C') {
			zoomPanel.newScale(zoomPanel.scale + .1);
		}
		
		if (key == 'F') {
			double newZoomScale = zoomPanel.scale - .1;
			if (newZoomScale < 1) { newZoomScale = 1; }
			zoomPanel.newScale(newZoomScale);
		}
		
		if (key == '.') { Env.scaleBarLength += Env.scaleBarInc; }
		if (key == ',') {
			Env.scaleBarLength += -Env.scaleBarInc;
			if (Env.scaleBarLength < 0.0) { Env.scaleBarLength = 0; }
		}
		
		if (key == 'Y') { 
			if (Env.lisCircleOn) { Env.lisCircleOn = false; } else { Env.lisCircleOn = true; }
		}
		
		
	}
	
	public void makeMenuBar () {
		MenuBar theMenuBar = new MenuBar();
		this.setMenuBar( theMenuBar );

		theMenuBar.add(Info.infoMenu);
		
	}
	
	public void makeMagSlider () {
		magSlider = new JSlider (100,50000,(int)(zoomPanel.scale*100));
		magSlider.setOrientation(JSlider.VERTICAL);
		//magSlider.setMajorTickSpacing(19900);
		//magSlider.setMinorTickSpacing(1000);
		//magSlider.setPaintTicks(true);
		//magSlider.createStandardLabels(19900);
		//magSlider.setPaintLabels(true);
		magSlider.addChangeListener(this);
	}
	
	public void makeSliderBox () {
		Font sliderFont = new Font ("TimesRoman", Font.PLAIN, 9);
		viewDistSlider = new JSlider (0,50,10);
		viewDistSlider.setOrientation(JSlider.HORIZONTAL);
		viewDistSlider.addChangeListener(this);
	
		focalSlider = new JSlider (0,200,80);
		focalSlider.setOrientation(JSlider.HORIZONTAL);
		focalSlider.addChangeListener(this);
		
		eyeSepSlider = new JSlider (-100,100,0);
		eyeSepSlider.setOrientation(JSlider.HORIZONTAL);
		eyeSepSlider.addChangeListener(this);
		
		eyeAngSlider = new JSlider (-200,200,17);
		eyeAngSlider.setOrientation(JSlider.HORIZONTAL);
		eyeAngSlider.addChangeListener(this);
		
		viewDist = new JLabel();
		viewDist.setFont(sliderFont);
		viewDist.setForeground(Color.black);
		viewDist.setBackground(Color.black);
		
		focalLength = new JLabel();
		focalLength.setFont(sliderFont);
		focalLength.setForeground(Color.black);
		focalLength.setBackground(Color.black);
		
		eyeSep = new JLabel();
		eyeSep.setFont(sliderFont);
		eyeSep.setForeground(Color.black);
		eyeSep.setBackground(Color.black);
		
		eyeAng = new JLabel();
		eyeAng.setFont(sliderFont);
		eyeAng.setForeground(Color.black);
		eyeAng.setBackground(Color.black);
		
		sliderBox.setLayout(new GridLayout(2,2,40,-2));
		
		JPanel viewDistBox = new JPanel (new GridLayout(1,2,5,0));
		viewDistBox.add(viewDist);
		viewDistBox.add(viewDistSlider);
		sliderBox.add(viewDistBox);
		
		JPanel focalBox = new JPanel (new GridLayout(1,2,5,0));
		focalBox.add(focalLength);
		focalBox.add(focalSlider);
		sliderBox.add(focalBox);
		
		JPanel eyeSepBox = new JPanel (new GridLayout(1,2,5,0));
		eyeSepBox.add(eyeSep);
		eyeSepBox.add(eyeSepSlider);
		sliderBox.add(eyeSepBox);
		
		JPanel eyeAngBox = new JPanel (new GridLayout(1,2,5,0));
		eyeAngBox.add(eyeAng);
		eyeAngBox.add(eyeAngSlider);
		sliderBox.add(eyeAngBox);
	}
		
	public void makeInfoBox () {
		Font infoFont = new Font ("TimesRoman", Font.PLAIN, 10);
		displayAngle = new JLabel();						// JLabel to show current X-Z viewing angle
		displayAngle.setFont(infoFont);
		displayAngle.setForeground(Color.black);
		displayAngle.setBackground(Color.black);
		
		unitVector = new JLabel();
		unitVector.setFont(infoFont);
		unitVector.setForeground(Color.black);
		unitVector.setBackground(Color.black);
		
		zoomCenter = new JLabel();
		zoomCenter.setFont(infoFont);
		zoomCenter.setForeground(Color.black);
		zoomCenter.setBackground(Color.black);
		
		infoBox.setLayout(new GridLayout(3,1));
		infoBox.add(displayAngle);
		infoBox.add(unitVector);
		infoBox.add(zoomCenter);
		updateInfo();
	}
	
	public void makePlayBox () {
		//playIcon = new ImageIcon("playerGifs/play.gif");
		//pauseIcon = new ImageIcon("playerGifs/pause.gif");
		//JButton playButton = new JButton(playIcon);
		JButton playButton = new JButton("->");
		this.playButton = playButton;
		playButton.setToolTipText("PLAY");
		playButton.setActionCommand("Play");
		playButton.addActionListener(this);
		
		//stopIcon = new ImageIcon("playerGifs/stop.gif");
		//stoppedIcon = new ImageIcon("playerGifs/stopped.gif");
		//JButton stopButton = new JButton(stoppedIcon);
		JButton stopButton = new JButton("");
		this.stopButton = stopButton;
		stopButton.setToolTipText("STOP");
		stopButton.setActionCommand("StopPlay");
		stopButton.addActionListener(this);
		
		//Icon nextIcon = new ImageIcon("playerGifs/next.gif");
		//JButton nextButton = new JButton(nextIcon);
		JButton nextButton = new JButton(">>");
		nextButton.setToolTipText("Next Frame");
		nextButton.setActionCommand("NextFrame");
		nextButton.addActionListener(this);
		
		//Icon prevIcon = new ImageIcon("playerGifs/prev.gif");
		//JButton prevButton = new JButton(prevIcon);
		JButton prevButton = new JButton("<<");
		prevButton.setToolTipText("Previous Frame");
		prevButton.setActionCommand("PreviousFrame");
		prevButton.addActionListener(this);
		
		JButton plotButton = new JButton("Plot");
		plotButton.setToolTipText("Plot");
		plotButton.setActionCommand("Plot");
		plotButton.addActionListener(this);
		
		playBox.setLayout(new GridLayout(1,4));
		playBox.add(prevButton);
		playBox.add(stopButton);
		playBox.add(playButton);
		playBox.add(nextButton);
		playBox.add(plotButton);
	}
		
	
	public void makeControlBox () {
		rotateButton = new JButton("rotate");
		rotateButton.setFont(Env.listFont);
		rotateButton.setActionCommand ("rotate");
		rotateButton.setToolTipText ("Rotate Viewing Angle");
		rotateButton.addActionListener(this);
		
		
		movieButton = new JToggleButton("movie");
		movieButton.setSelected(false);
		movieButton.setActionCommand ("makeMovie");
		movieButton.setFont(Env.listFont);
		movieButton.setToolTipText ("Make A Movie");
		movieButton.addActionListener(this);
		
		trackActinB = new JToggleButton("track actin");
		trackActinB.setSelected(false);
		trackActinB.setActionCommand ("trackActin");
		trackActinB.setFont(Env.listFont);
		trackActinB.setToolTipText ("track actin density");
		trackActinB.addActionListener(this);
		
		eyesIcon = new ImageIcon("playerGifs/eyes.gif");
		eyesOnIcon = new ImageIcon("playerGifs/eyesOn.gif");
		eyesButton = new JButton(eyesIcon);
		eyesButton.setActionCommand("PinHole View");
		eyesButton.setToolTipText ("Use Pinhole Effects");
		eyesButton.addActionListener(this);
		
		String [] actinOptions = {"all", "spheres", "helices","lines"};
		actinOptionsBox = new JComboBox(actinOptions);
		actinOptionsBox.setSelectedIndex(0);
		actinOptionsBox.setFont(Env.listFont);
		actinOptionsBox.setActionCommand("actinOptions");
		actinOptionsBox.setToolTipText("actin drawing style");
		actinOptionsBox.addActionListener(this);
		
		
		paintButton = new JToggleButton("paint");
		paintButton.setSelected(true);
		paintButton.setActionCommand("paintOff");
		paintButton.setFont(Env.listFont);
		paintButton.setToolTipText ("Don't show frames on screen");
		paintButton.addActionListener(this);
		
		JButton inButton = new JButton("zoom+");
		inButton.setActionCommand("zoomIn");
		inButton.setFont(Env.listFont);
		inButton.setToolTipText ("Zoom In");
		inButton.addActionListener(this);
		
		JButton outButton = new JButton("zoom-");
		outButton.setActionCommand("zoomOut");
		outButton.setFont(Env.listFont);
		outButton.setToolTipText ("Zoom Out");
		outButton.addActionListener(this);
		
		JButton skipPlus = new JButton("skip+");
		skipPlus.setActionCommand("skipPlus");
		skipPlus.setFont(Env.listFont);
		skipPlus.setToolTipText ("Skip more frames");
		skipPlus.addActionListener(this);
		
		JButton skipMinus = new JButton("skip-");
		skipMinus.setActionCommand("skipMinus");
		skipMinus.setFont(Env.listFont);
		skipMinus.setToolTipText ("Skip less frames");
		skipMinus.addActionListener(this);
		
		JLabel scaleLabel = new JLabel("Scale=");
		//scaleLabel.setFont(Env.listFont);
		scaleLabel.setHorizontalAlignment(JLabel.RIGHT);
		
		scaleField = new JTextField(String.valueOf(zoomPanel.scale));
		scaleField.setToolTipText("manually set scale here");
		//scaleField.setFont(Env.listFont);
		scaleField.setBackground(this.getBackground());
		scaleField.setActionCommand("manualScaleSet");
		scaleField.addActionListener(this);
		
		controlBox.setLayout(new GridLayout(2,6));
		//controlBox.add(rotateButton);
		controlBox.add(movieButton);
		controlBox.add(trackActinB);
		//controlBox.add(eyesButton);
		controlBox.add(paintButton);
		controlBox.add(scaleLabel);
		controlBox.add(scaleField);
		controlBox.add(actinOptionsBox);
		controlBox.add(skipPlus);
		controlBox.add(skipMinus);
		controlBox.add(inButton);
		controlBox.add(outButton);
		
	}
	
	public void makeNorthPanel () {
		northPanel.setLayout(new GridLayout(1,1));
		northPanel.add(playBox);
		northPanel.add(moviePath);
	}
	
	public void makeSouthPanel () {
		southPanel.setLayout(new FlowLayout());
		southPanel.add(infoBox);
		southPanel.add(controlBox);
		
	}
	
	public void makePopUp () {
		analysisMenu = new JPopupMenu();
		analysisMenu.setInvoker(this);
		analysisMenu.add ("**Other Analysis**");
		JMenuItem plotVars = new JMenuItem("Plot");
		plotVars.addActionListener(this);
		JMenuItem stepsA = new JMenuItem("Batch Histograms");
		stepsA.addActionListener(this);
		JMenuItem otherConfig = new JMenuItem("Pairwise Histogram");
		otherConfig.addActionListener(this);
		JMenuItem epsOutItem = new JMenuItem("EPS Out");
		epsOutItem.setActionCommand("epsOut");
		epsOutItem.addActionListener(this);
		analysisMenu.add(plotVars);
		analysisMenu.add(stepsA);
		analysisMenu.add(otherConfig);
		analysisMenu.add(epsOutItem);
	}
	
	private void talkln (String info) {
		System.out.println(info);
	}
}
