package rocketBugsRender;
/* 
	PlotProps ... plotting properties for a given DataColumn
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.text.*;
import java.util.*;

public class XLimits extends JFrame implements ChangeListener, ActionListener, WindowListener, ComponentListener, KeyListener, MouseListener, MouseMotionListener, AdjustmentListener {
	PlotFrame parentFrame;
	JPanel titlePanel;
	JPanel continuePanel;
	JPanel plotOptPanel;
	JComboBox plotTypeBox, textTypeBox;
	JLabel titleLabel,minLabel,maxLabel;
	JTextField minField,maxField;
	JButton plotButton;		// a link to button that will update the plot
	boolean fieldError = false;
	
	// plotting properties to carry
	double min, max;

	// text formats
	int decPtCt = 2;
	DecimalFormat expFormat = new DecimalFormat ("0.00E0");
	DecimalFormat decFormat = new DecimalFormat ("0.00");
	DecimalFormat intFormat = new DecimalFormat ("0");
	DecimalFormat textFormat = expFormat;

	public XLimits (PlotFrame parentFrame) {
		this.parentFrame = parentFrame;
		
		makeTitlePanel();
		makeContinuePanel();
		makePlotOptPanel();
		
		setSize(230,130);
		
		addKeyListener(this);
		addComponentListener(this);
		addMouseListener(this);
		addMouseMotionListener(this);
		addWindowListener(this);
		
		initialize();
		show();
	}
	
	public void initialize () {
		getContentPane().removeAll();
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add (titlePanel, BorderLayout.NORTH);
		getContentPane().add (plotOptPanel, BorderLayout.CENTER);
		getContentPane().add (continuePanel, BorderLayout.SOUTH);
	
	}
			
	
	public void getProps (JButton pB) {
		minField.setText(String.valueOf(min));
		maxField.setText(String.valueOf(max));
		plotButton = pB;
		show();
	}
	
	public void propsFromGuiCancel () {
		
	}
	
	public void propsFromGui () {
		Double DD = Double.valueOf(minField.getText());
		min = DD.doubleValue();
		if (Double.isNaN(min)) { 
			minField.setForeground(Color.red);
			fieldError = true;
		} else {
			minField.setForeground(Color.black);
		}
		
		DD = Double.valueOf(maxField.getText());
		max = DD.doubleValue();
		if (Double.isNaN(min)) { 
			maxField.setForeground(Color.red);
			fieldError = true;
		} else {
			maxField.setForeground(Color.black);
		}		
		if (min > max) {  // safe for incorrectly ordered min and max
			double swapT = min;
			min = max;
			max = swapT;
		}
	}

	
	public void actionPerformed (ActionEvent e) {
		String act = e.getActionCommand();
		
		if (act.equals("Cancel")) {
			propsFromGuiCancel();
			hide();
		}
		
		if (act.equals("Apply")) {
			fieldError = false;
			propsFromGui();
			if (!fieldError) { 
				parentFrame.rowRange[0] = parentFrame.xColumn.getClosestDataLoc(min);
				parentFrame.rowRange[1] = parentFrame.xColumn.getClosestDataLoc(max);
				parentFrame.plotAll();
			}
			
		}
		
		
	}
	
	public void stateChanged(ChangeEvent e) {
        JSlider source = (JSlider)e.getSource();
    }
	
	public void adjustmentValueChanged (AdjustmentEvent e) {
	}
	
	
	public void windowActivated (WindowEvent e) {}
	public void windowClosing (WindowEvent e) {}
	public void windowDeactivated (WindowEvent e) {}
	public void windowDeiconified (WindowEvent e) {}
	public void windowIconified (WindowEvent e) {}
	public void windowOpened (WindowEvent e) {}
	public void windowClosed (WindowEvent e) {}
	
	public void keyReleased (KeyEvent e) {}
	public void keyTyped (KeyEvent e) {}
	public void keyPressed (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
	}
	
	public void mouseClicked (MouseEvent e) {}
	public void mouseEntered (MouseEvent e) {}
	public void mouseExited (MouseEvent e) {}
	public void mousePressed (MouseEvent e) {
		int xPos = e.getX();
		int yPos = e.getY();
	}
	
	public void mouseReleased (MouseEvent e) {}
	public void mouseDragged (MouseEvent e) {}
	public void mouseMoved (MouseEvent e) {}
	
	public void componentHidden (ComponentEvent e) {}
	public void componentShown (ComponentEvent e) {}
	public void componentMoved (ComponentEvent e) {}
	public void componentResized (ComponentEvent e) {}
	
	public void makeTitlePanel () {
		titlePanel = new JPanel();
		titleLabel = new JLabel("X Axis Limits");
		titlePanel.add(titleLabel);
	}
	
	public void makePlotOptPanel() {
		plotOptPanel = new JPanel();
		
		JLabel dummyLabel = new JLabel("");
		
		// text field dimension
		Dimension textDim = new Dimension(50,20);
		
		minLabel = new JLabel("Min. axis value:");
		minLabel.setFont(Env.controlFont);
		minLabel.setEnabled(true);

		minField = new JTextField(6);
		minField.setFont(Env.controlFont);
		minField.setPreferredSize(textDim);
		minField.setEnabled(true);
		
		maxLabel = new JLabel("Max. axis value:");
		maxLabel.setFont(Env.controlFont);
		maxLabel.setEnabled(true);
		
		maxField = new JTextField(6);
		maxField.setFont(Env.controlFont);
		maxField.setPreferredSize(textDim);
		maxField.setEnabled(true);
		
		plotOptPanel.setLayout(new GridLayout(2,2));
		plotOptPanel.add(minLabel);
		plotOptPanel.add(minField);
		plotOptPanel.add(maxLabel);
		plotOptPanel.add(maxField);
	}

	public void makeContinuePanel () {
		continuePanel = new JPanel();
	
		JButton cancelB = new JButton ("Cancel");
		cancelB.setActionCommand("Cancel");
		cancelB.setToolTipText("Cancel Changes");
		cancelB.addActionListener(this);
		
		JButton setB = new JButton("Apply");
		setB.setActionCommand("Apply");
		setB.setToolTipText("Apply Changes");
		setB.addActionListener(this);
		
		continuePanel.setLayout(new FlowLayout());
		continuePanel.add(cancelB);
		continuePanel.add(setB);
	}

	private void talk (String info) {
		System.out.print(info);
	}
	
	private void talkln (String info) {
		System.out.println(info);
	}
	

}